#!/usr/bin/python2.7
"""Maven-related utilities for java packages."""

import re


def GetMavenArtifactId(api_name, package_path='', canonical_name='',
                       owner_domain='google.com'):
  """Returns the maven artifact id for a given api.

  Args:
    api_name:  (str) The api name.
    package_path: (str|None) The package path, if any.
    canonical_name: (str|None) The canonical api name, if any.
    owner_domain: (str) The api's owner domain.
  Returns:
    (str) The artifact id.
  """
  # TODO(user): This logic is meant to match equivalent
  # logic in api.Api._SetupModules() in setting up the path,
  # using canonicalName only if packagePath is set, for backwards
  # compatibility.  A TODO there proposes a breaking change of
  # obeying canonicalName unconditionally; were that done this
  # would have to change likewise for our tests to pass.
  if package_path and canonical_name:
    api_name = canonical_name.lower().replace(' ', '')

  parts = []
  if owner_domain == 'google.com':
    parts.extend(['google', 'api', 'services'])
  if package_path:
    parts.extend(re.split(r'\.|/', package_path))
  parts.append(api_name)
  return '-'.join(parts)


def GetMavenGroupId(owner_domain):
  """Returns the maven group id for a given owner domain.

  Args:
    owner_domain: (str) The owner domain.
  Returns:
    (str) The group id.
  """
  if owner_domain == 'google.com':
    return 'com.google.apis'
  else:
    return '.'.join(reversed(owner_domain.split('.')))


def GetMavenVersion(api, language_version):
  """Returns the maven version."""
  if api.get('ownerDomain') == 'google.com':
    return '%s-rev%s-%s' %(api['version'],
                           api['revision'],
                           language_version)
  return '%s-%s-SNAPSHOT' % (api['version'], language_version)


def GetMavenMetadata(api, language_version):
  """Returns a dict of useful maven metadata."""
  owner_domain = api.get('ownerDomain', 'google.com')
  return {
      'artifact_id': GetMavenArtifactId(
          api['name'], api.get('packagePath'),
          api.get('canonicalName'), owner_domain),
      'group_id': GetMavenGroupId(owner_domain),
      'version': GetMavenVersion(api, language_version),
      }



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
# Copyright 2011 Google Inc. All Rights Reserved.

"""Assorted utility methods for the code generator."""

__author__ = 'aiuto@google.com (Tony Aiuto)'


import re


_WORD_SPLIT_PATTERN = re.compile(r'[\._/-]+')


def CamelCase(s):
  """CamelCase a string so that it is more readable as a variable name.

  Camelcases a string, begining new words after any instances of '.', '_',
      '/', or '-'.

  Args:
    s: (str) A string.
  Returns:
    s, with the first letter of each word capitalized.
  """
  title = lambda x: x[0].upper() + x[1:] if x else x
  return ''.join([title(x) for x in _WORD_SPLIT_PATTERN.split(s)])


def UnCamelCase(phrase, separator='_'):
  """Convert CamelCased phrase into lower-case delimited words.

  Args:
    phrase: CamelCased phrase.
    separator: The word separator to inject between lowercased words.
  Returns:
    lower case phrase with separators between case changes from lower
    to upper or acronyms (all upper) to lower case.
  """
  phrase_len = len(phrase)
  if not phrase_len:
    return ''

  ch = phrase[0]
  text_run = ch.isalnum()
  last_was_separator = ch.isupper() or not text_run
  caps_run = False
  result = ch.lower()

  # already did first index
  for i in range(phrase_len - 1):
    ch = phrase[i + 1]
    if ch.isupper():
      caps_run = text_run and last_was_separator
      text_run = True
      if not last_was_separator:
        result += separator
        last_was_separator = True
    elif not ch.isalnum():
      caps_run = False
      text_run = False
      last_was_separator = True
    else:
      text_run = True
      last_was_separator = False
      if caps_run:
        result += separator
        last_was_separator = True
        caps_run = False

    result += ch.lower()
  return result


def SanitizeDomain(s):
  """Sanitize a domain name to ch aracters suitable for use in code.

  We only want text characters, digits, and '.'. For now, we only allow ASCII,
  characters but we may revisit that in the future if there is demand from
  Endpoints customers.

  Since the pattern 'my-custom-app.appspot.com' is a popular one, preserve the
  '-' in a useful way.

  Args:
    s: (str) A domain name
  Returns:
    (str) A version of the domain name suitable for use in code structures
        like Java package names. None if s is None.
  """
  if s is None:
    return None
  s = s.lower().replace('-', '_')
  return ''.join([c for c in s if c.isalnum() or c in ['.', '_']])


def ReversedDomainComponents(s):
  """Returns a list of domain components in reverse order.

  Args:
    s: (str) A string of the form "a.b.c"
  Returns:
    list(s) E.g. ['c', 'b', 'a']
  """
  if not s:
    return []
  parts = s.split('.')
  parts.reverse()
  return parts


def NoSpaces(s):
  """Remove spaces from a string, but preserves None-ness."""
  if s:
    return s.replace(' ', '')
  return s



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

r"""Command line tool to run the Google APIs template expander.

This tool takes a triplet of:
  - an API discovery document
  - a language (and variant)
  - a template tree name
and expands the template tree in the context of that API and tree.

Usage:
  expand_templates.py \
    --discovery=myapi.json \
    --language=python \
    --templates=my_templates \
    --output_dir=my_expanded_templates
"""

__author__ = 'aiuto@google.com (Tony Aiuto)'

import json
import os

from google.apputils import app
import gflags as flags
from googleapis.codegen import generator
from googleapis.codegen import generator_lookup
from googleapis.codegen import language_model
from googleapis.codegen.api import Api
from googleapis.codegen.filesys import package_writer_foundry
from googleapis.codegen.targets import Targets

FLAGS = flags.FLAGS

flags.DEFINE_string(
    'discovery',
    None,
    'A discovery document captured from a discovery service.')
flags.DEFINE_boolean(
    'include_timestamp',
    False,
    'Adds a timestamp to the generated source files.')
flags.DEFINE_enum(
    'language',
    'any',
    ['any'] + generator_lookup.SupportedLanguages(),
    'Target language for the generated code')
flags.DEFINE_string(
    'language_variant',
    'default',
    'which variant of "language" to generate for. E.g. "stable" vs. "head".')
flags.DEFINE_string(
    'output_dir',
    None,
    'A path to a directory where the generated files will be created.')
flags.DEFINE_string(
    'output_file',
    None,
    'An output file path to contain the archive for the generated library.'
    ' The contents of the file are determined by the output_format parameter')
flags.DEFINE_enum(
    'output_format',
    'zip',
    ['zip', 'tgz', 'tar'],
    'What format to use for --output_file.')
flags.DEFINE_enum(
    'output_type',
    'plain',
    ['plain', 'full'],
    'What kind of output to make.'
    ' plain=just the source,'
    ' full=turn on all the optional parts (useful for testing the generator).'
    )
flags.DEFINE_string(
    'templates',
    None,
    'The name of the template suite (w.r.t. language and variant)')
flags.DEFINE_bool('version_package', False, 'Put API version in package paths')

flags.declare_key_flag('discovery')
flags.declare_key_flag('include_timestamp')
flags.declare_key_flag('language')
flags.declare_key_flag('language_variant')
flags.declare_key_flag('output_dir')
flags.declare_key_flag('output_file')
flags.declare_key_flag('output_format')
flags.declare_key_flag('output_type')
flags.declare_key_flag('templates')
flags.declare_key_flag('version_package')


def main(unused_argv):
  if not FLAGS.discovery:
    raise app.UsageError('You must specify --discovery')
  if not (FLAGS.output_dir or FLAGS.output_file):
    raise app.UsageError(
        'You must specify one of --output_dir or --output_file')
  if not FLAGS.templates:
    raise app.UsageError('You must specify --templates')

  f = open(FLAGS.discovery)
  discovery_doc = json.loads(f.read())
  f.close()

  options = {
      # Include other files needed to compile (e.g. base jar files)
      'include_dependencies': False,
      # Include the timestamp in the generated library
      'include_timestamp': FLAGS.include_timestamp,
      # Put API version in the package
      'version_package': FLAGS.version_package,
      }
  if FLAGS.output_type == 'full':
    options['include_dependencies'] = True

  # try to determine what generator to use from the language variant.
  language_variants = Targets().VariationsForLanguage(FLAGS.language)
  if language_variants:
    features = language_variants.GetFeatures(FLAGS.language_variant)
    template_dir = features.template_dir
    generator_name = features.get('generator', FLAGS.language)
  else:
    # Not described by targets.json. The template dir should just be down
    # in the language specific folder
    template_dir = os.path.join(os.path.dirname(__file__), 'languages',
                                FLAGS.language)
    features = None
    generator_name = FLAGS.language

  # Instantiate the right code generator
  lang_model = None
  try:
    if FLAGS.language == 'any':
      api = Api(discovery_doc)
      # TODO(user): A default language model should be built in to the
      #   templates
      lang_model = language_model.DocumentingLanguageModel()
    else:
      generator_class = generator_lookup.GetGeneratorByLanguage(generator_name)
      generator_instance = generator_class(discovery_doc)
      api = generator_instance.api
      lang_model = generator_instance.language_model

  except ValueError:
    raise app.UsageError('Unsupported language option: %s' % FLAGS.language)

  api.VisitAll(lambda o: o.SetLanguageModel(lang_model))
  gen = TemplateExpander(api, options=options)

  if features:
    gen.SetFeatures(features)

  template_dir = os.path.join(template_dir, FLAGS.templates)
  if not os.path.isdir(template_dir):
    raise app.UsageError('Can not find template tree at: %s' % template_dir)
  gen.SetTemplateDir(template_dir)

  # Get an output writer
  package_writer = package_writer_foundry.GetPackageWriter(
      output_dir=FLAGS.output_dir, output_file=FLAGS.output_file,
      output_format=FLAGS.output_format)

  # do it
  gen.GeneratePackage(package_writer)
  package_writer.DoneWritingArchive()
  return 0


class TemplateExpander(generator.TemplateGenerator):

  def __init__(self, api, options=None):
    super(TemplateExpander, self).__init__(options=options)
    self._api = api

  def GeneratePackage(self, package_writer):
    """Walk the templates and generate output.

    Overrides the default.

    Args:
      package_writer: (LibraryPackage) output package
    """
    path_replacements = {
        '___package___': self._api.module.path
        }
    variables = {
        'api': self._api,
        'options': self._options,
        }
    self.WalkTemplateTree('.', path_replacements, {}, variables, package_writer)


if __name__ == '__main__':
  app.run()



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""A LibraryPackage that creates a Zip file.

This module aids in the construction of a ZIP file containing all the
components generated and required by a library.
"""

__author__ = 'aiuto@google.com (Tony Aiuto)'

import os
import StringIO
import zipfile

from googleapis.codegen.filesys.library_package import LibraryPackage


class ZipLibraryPackage(LibraryPackage):
  """The library package."""

  def __init__(self, stream):
    """Create a new ZipLibraryPackage.

    Args:
      stream: (file) A file-like object to write to.
    """
    super(ZipLibraryPackage, self).__init__()
    self._zip = zipfile.ZipFile(stream, 'w', zipfile.ZIP_STORED)
    self._current_file_data = None
    self._created_dirs = []  # list of directory entries we have auto-created

  def StartFile(self, name):
    """Start writing a named file to the package.

    Args:
      name: (str) path which will identify the contents in the archive.

    Returns:
      A file-like object to write the contents to.
    """
    self.EndFile()
    self._current_file_data = StringIO.StringIO()
    name = '%s%s' % (self._file_path_prefix, name)
    # Let this explode if the name is not ascii.
    self._current_file_name = name.encode('ascii')
    self.CreateDirectory(os.path.dirname(self._current_file_name))
    return self._current_file_data

  def CreateDirectory(self, directory):
    """Create one or more directory entries, esssentially mkdir -p.

    Some zip readers do not create needed folders for arbitrary file paths.
    When we encounter a new path for the first time, we make sure the tree
    exists.

    Args:
      directory: (str) a directory name.
    """
    # Build list of dirs to build. This will be from bottom up.
    to_create = []
    while directory:
      if directory in self._created_dirs:
        break
      to_create.append(directory)
      self._created_dirs.append(directory)
      directory = os.path.dirname(directory)
    to_create.reverse()  # have to build from top down
    for directory in to_create:
      # Create the directory entry.  File and directory names must be
      # ascii.
      info = zipfile.ZipInfo((directory + '/').encode('ascii'),
                             date_time=self.ZipTimestamp())
      # Notes from the web and zipfile sources:
      # external_attr is 32 in size, with the unix permissions in the
      # high order 16 bit, and the MS-DOS FAT attributes in the lower 16.
      # man 2 stat tells us that 040755 should be a drwxr-xr-x style file,
      # and word of mouth tells me that bit 4 marks directories in FAT.
      info.external_attr = (040755 << 16) | 0x10
      self._zip.writestr(info, '')

  def EndFile(self):
    """Flush the current output file to the ZIP container."""
    if self._current_file_data:
      info = zipfile.ZipInfo(self._current_file_name,
                             date_time=self.ZipTimestamp())
      # This is a chmod 0644, but you have to read the zipfile sources to know
      info.external_attr = 0644 << 16
      data = self._current_file_data.getvalue()
      # File contents may be utf-8
      if isinstance(data, unicode):
        data = data.encode('utf-8')
      self._zip.writestr(info, data)
      self._current_file_data.close()
      self._current_file_data = None

  def ZipTimestamp(self):
    # Use a constant timestamp to avoid non-deterministic build-time output.
    return (1980, 1, 1, 0, 0, 1)

  def DoneWritingArchive(self):
    """Signal that we are done writing the entire package.

    This method must be called to flush the zip file directory to the output
    stream.
    """
    if self._zip:
      self.EndFile()
      self._zip.close()
      self._zip = None

  def FileExtension(self):
    """Returns the file extension for this archive, which is zip."""
    return 'zip'

  def MimeType(self):
    """Returns the MIME type for this archive."""
    return 'application/zip'



#!/usr/bin/python2.7
# Copyright 2012 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""C++ library generator.

Specializations to the code Generator for C++ bindings.
"""

__author__ = 'ewiseblatt@google.com (Eric Wiseblatt)'

import re
import gflags as flags
from googleapis.codegen import api
from googleapis.codegen import api_library_generator
from googleapis.codegen import cpp_import_manager
from googleapis.codegen import data_types
from googleapis.codegen import language_model
from googleapis.codegen import utilities
from googleapis.codegen.import_definition import ImportDefinition

FLAGS = flags.FLAGS
flags.DEFINE_boolean(
    'cpp_generator_add_owner_dir',
    True,
    'Adds owner subdirectory to --output_dir path.')

# This pattern is used to extract the DT and DD text groups.
# It is meant to be applied to the split partions of the dl_pattern_re.
# It turns  'A - Description A\nB - Description B'
#     into  ['', 'A', 'Description A', '', 'B', 'Description B', '']
DT_DD_PATTERN_RE = re.compile(
    r"""\s*-?\s?   # Skip leading spaces and leading - decorator from the key.
        (\S+)      # Capture the (single-word) key.
        \s*[\-\:]   # There must be a '-' or ':' after the key, but
                   # can be surrounded by spaces, even on different line.
        \s*        # The - or : seperator can be followed by spaces (or eoln).
       ([^\n]+)    # The value is everything to eoln.
                   # We already prefiltered the length, so dont worry about it.
       """,
    re.VERBOSE)

# This threshold is somewhat arbitrary -- two 80 character lines.
# We dont want the dd elements to be bigger than this or else it probably
# wasnt a dd. If the very last line was cut off at this length then we'll
# assume that it is runoff and the list was not terminated with an EOL.
# In that case we'll back off and just keep the first sentence for the DD
# and stick the remainder after the DL closes along with the rest of the
# text.
MAX_DD_LEN = 160

# This pattern is used to extract the content for a DL.
# It will be the entire string for all the individual element definitions.
# It turns  'Plain text\nMore plain text'
#      into ['Plain text\nMore plain text']
# and turns 'Before\nA - Description A\nB - Description B\nAfter'
#      into ['Before', 'A - Description A\nB - Description B', '\nAfter']
#                      +--------  DL list contents  --------+
_DL_PATTERN = r"""\n
        (         # Capture group for the whole matched RE.
        (?:       # Non-capture group for the DT/DD name/value pair.
        \s*-?\s?  # Skip leading spaces and single dash starting the line.
        \S+       # The key must be a single word without any spaces.
        \s?[\-\:] # The key can have a space (and even eoln)
                  # But then must have a dash or colon
        \s*       # Can have spaces or eoln after the dash/colon
        [^\n]{5,%d}  # The value part needs to be 5..N chars up to EOLN
        )         # That's the end of this DT/DD pair.
        {2,}      # To have a DL, we need at least 2 DT/DD pairs.
        \n?       # strip trailing space because we'll be adding it back in
                  # when we join with the next fragment.
        )         # End the capture group for all the tuples in the list.
        """ % MAX_DD_LEN
DL_PATTERN_RE = re.compile(_DL_PATTERN, re.VERBOSE)


class CppGenerator(api_library_generator.ApiLibraryGenerator):
  """The C++ code generator."""

  # pylint: disable=redefined-outer-name
  def __init__(self, discovery, language='cpp', language_model=None,
               options=None):
    if not language_model:
      language_model = CppLanguageModel(options)
    super(CppGenerator, self).__init__(CppApi, discovery, language,
                                       language_model, options=options)

  def _InnerModelClassesSupported(self):
    """Gets whether or not inner model classes are supported."""
    return True

  def _HeaderFileName(self, path, filename):
    if not path:
      return '%s.h' % filename
    return '%s/%s.h' % (path, filename)

  def _DLifyDescription(self, text):
    """Substitutes <DL> list into text based on patterns.

    Args:
      text: (string) the text to transform is assumed to be HTML escaped.

    Returns:
      Patterns that look like DLl lists will be replaced with HTML markup.
    """

    # Mark beginning of string with \n so we can catch things at the beginning
    # since we require a \n at the start of the first key.
    sentinel = '\n'

    # Look for pattern in the form \n<term> - <description>\n and extract
    # them out into a <dl> list.
    # Note that descriptions are normally HTML safe, but that check already
    # happened so this will actually produce HTML, which we want.

    # We'll partition the text by the DL lists.
    dl_partitions = DL_PATTERN_RE.split(sentinel + text)  # , 0, re.MULTILINE)
    dl_partitions[0] = dl_partitions[0][len(sentinel):]
    if len(dl_partitions) == 1:
      return text

    # We'll iterate over the partitions and attempt to split the DT/DD pairs.
    # If we cannot then the partition was between the lists in the split.
    # If we can, then we'll use that split to form the lines in the DL.
    result = []
    for partition in dl_partitions:
      dl = DT_DD_PATTERN_RE.split(partition)
      if len(dl) == 1:
        if dl[0]:  # skip empty strings
          result.append(dl[0])
          continue
      else:
        # we have a list in the form <seperator>, <dt value>, <dd value>
        # so iterate over those 3-group pairs and extract out the dt nd dd
        # values, adding them into a dl.
        # I'm calling these triples rather than 3-tupels because they are
        # not actually grouped as tuples.
        if result:
          result.append('\n')
        num_triples = len(dl) / 3
        if num_triples == 1:
          result.append(partition)
          continue

        runoff = ''  # Extra stuff past end of dl that regex cannot detect.
        result.append('<dl>\n')
        for i in range(num_triples):
          dt_value = dl[3 * i + 1].strip()
          dd_value = dl[3 * i + 2].strip()
          result.append('<dt>%s\n' % dt_value)
          if i < (num_triples - 1) or len(dd_value) < MAX_DD_LEN:
            # Make sure that dd's end with periods
            # but dont require dt's to.
            if not dd_value.endswith('.'):
              dd_value += '.'
            result.append('<dd>%s\n' % dd_value)
          else:
            period = dd_value.find('.')
            if period < 0:
              period = len(dd_value) - 1
            result.append('<dd>%s\n' % dd_value[0:period + 1])
            runoff = dd_value[period + 1:]

        result.append('</dl>\n')
        result.append(runoff)
    return ''.join(result)

  def AnnotateDocumentation(self, code_obj):
    """Adds properties used to generate better documentation.

    Args:
      code_obj: (CodeObject) The entity to annotate
    """

    description = code_obj.values.get('description')
    if not description:
      return

    # Trailing EOLN should be stripped already.
    if description.endswith('\n'):
      description = description[0:len(description) - 1]

    # Make sure description ends with a period.
    if not description.endswith('.'):
      description += '.'

    # Check for mention about being deprecated.
    if re.match(r'(?:^|\W)deprecated(?:$|\W)', description, re.IGNORECASE):
      code_obj.SetTemplateValue('deprecated', True)

    description = self._DLifyDescription(description)
    code_obj.SetTemplateValue('description', description)

  def AnnotateApiForLanguage(self, the_api):
    super(CppGenerator, self).AnnotateApiForLanguage(the_api)
    unsorted_list = the_api.TopLevelModelClasses()
    have = set()
    sorted_list = []
    for m in unsorted_list:
      self._AddDependencies(the_api, m, have, sorted_list)
    the_api.SetTemplateValue('sortedTopLevelModels', sorted_list)

  def AnnotateApi(self, the_api):
    """Annotate a Api with C++ specific elements."""
    super(CppGenerator, self).AnnotateApi(the_api)
    if the_api.values.get('version') and self._options.get('version_package'):
      the_api.module.SetPath('%s_%s' % (
          the_api.module.package_path, the_api.values.get('versionNoDots')))

    for schema in the_api.TopLevelModelClasses():
      filename = utilities.UnCamelCase(schema.class_name)
      schema.SetTemplateValue('filename', filename)
      schema.SetTemplateValue(
          'include_path',
          self._HeaderFileName(schema.module.path, schema.values['filename']))
      include_guard = '%s_%s_H_' % (the_api.module.name, filename)
      schema.SetTemplateValue('include_guard', include_guard.upper())

    the_api.SetTemplateValue('filename',
                             utilities.UnCamelCase(the_api.class_name))
    the_api.SetTemplateValue(
        'include_path',
        self._HeaderFileName(the_api.module.path, the_api.values['filename']))

    monolithic_source_name = the_api.GetTemplateValue('monolithicSourceName')
    if monolithic_source_name:
      the_api.SetTemplateValue('kitchensink', monolithic_source_name)
    else:
      plain_api_filename = utilities.UnCamelCase(the_api.values['name'])
      the_api.SetTemplateValue('kitchensink', '%s_api' % plain_api_filename)

    authscopes = the_api.values.get('authscopes')
    if authscopes:
      for scope in authscopes:
        self.AnnotateDocumentation(scope)

    self.AnnotateDocumentation(the_api)

  def _AddDependencies(self, the_api, model, have, dep_list):
    if model not in have:
      have.add(model)
      import_manager = cpp_import_manager.CppImportManager.ForElement(model)
      for dependent in import_manager.type_dependencies:
        schema = the_api.SchemaByName(dependent.values['className'])
        self._AddDependencies(the_api, schema, have, dep_list)
      dep_list.append(model)

  def AnnotateMethod(self, the_api, method, unused_rsrc):
    """Override the default."""
    super(CppGenerator, self).AnnotateMethod(the_api, method, unused_rsrc)
    import_manager = cpp_import_manager.CppImportManager.ForElement(the_api)
    request_type = method.values.get('requestType')
    response_type = method.values.get('responseType')
    if request_type:
      self._HandleImports(request_type, import_manager)

    if method.values.get('mediaUpload'):
      media_upload_h = '"googleapis/client/service/media_uploader.h"'
      import_manager.AddImport(media_upload_h)

    if response_type != the_api.void_type:
      request_pager_h = '"googleapis/client/service/service_request_pager.h"'
      self._HandleImports(method.values['responseType'], import_manager)
      if (method.values.get('isPageable') and
          method.values['isPagingStyleStandard']):
        import_manager.AddImport(request_pager_h)

    method.SetTemplateValue(
        'undecoratedClassName',
        '%sMethod' % utilities.CamelCase(method.values['wireName']))
    self.AnnotateDocumentation(method)

  def AnnotateParameter(self, unused_method, parameter):
    """Annotate a Parameter with C++ specific elements."""
    super(CppGenerator, self).AnnotateParameter(unused_method, parameter)
    parameter.SetTemplateValue('isPrimitive',
                               self._IsPrimitiveType(parameter.data_type))
    if parameter.data_type.code_type == 'string':
      string_type = self.features.get('stringType') or 'absl::string_view'
      parameter.SetTemplateValue('parameterCodeType', string_type)
    else:
      parameter.SetTemplateValue('parameterCodeType',
                                 parameter.data_type.code_type)
    self._HandleImports(parameter)
    self.AnnotateDocumentation(parameter)

  def AnnotateProperty(self, unused_api, prop, unused_schema):
    """Annotate a Property with C++ specific elements."""
    super(CppGenerator, self).AnnotateProperty(unused_api, prop, unused_schema)
    prop.SetTemplateValue('isPrimitive', self._IsPrimitiveType(prop.data_type))
    base_type = prop.data_type
    if base_type.GetTemplateValue('isContainer'):
      base_type = base_type.GetTemplateValue('baseType')

    try:
      has_parent = base_type.parent
    except AttributeError:
      has_parent = False

    if not base_type.GetTemplateValue('builtIn') and not has_parent:
      # Since the property's base type doesn't have a parent, the
      # base type may be safely used by a split accessor definition
      # at the top-level scope.
      #
      # Since the property's base type is not a built-in type, it
      # might be part of a set of mutually recursive types, which
      # can *only* be safely used by a split accessor definition at
      # the top-level scope, after all of the potentially recursive
      # class definitions have been emitted.
      #
      # So: we set the useSplitAccessor template value, causing C++
      # template expansion to declare the accessor methods for this
      # property as part of the object, but to define them separately
      # at the top-level scope.
      prop.SetTemplateValue('useSplitAccessor', True)

    self._HandleImports(prop)
    self.AnnotateDocumentation(prop)

  def AnnotateSchema(self, the_api, schema):
    """Annotate a Schema with C++ specific elements."""

    super(CppGenerator, self).AnnotateSchema(the_api, schema)
    # imports were already handled.

    self.AnnotateDocumentation(schema)

  def _IsPrimitiveType(self, data_type):
    """Determine if data type is representated as a C++ primitive type.

    Args:
      data_type: (DataType) The type to check.
    Returns:
      true if the data type is a primitive type (bool, float, double, int*)
      false otherwise (string, object, array, map).
    """
    return (data_type.json_type == 'integer'
            or data_type.json_type == 'number'
            or data_type.json_type == 'boolean'
            or (hasattr(data_type, 'json_format') and (
                data_type.json_format == 'int64'
                or data_type.json_format == 'uint64'
                or data_type.json_format == 'date'
                or data_type.json_format == 'date-time')))

  def _HandleImports(self, element, import_manager=None):
    """Handles imports for the specified element.

    Args:
      element: (Property|Parameter) The property we want to set the import for.
      import_manager: The import manager to import into if not the implied one.

    """
    # If this is a reference then resolve it so we can check the details.
    element = getattr(element, 'referenced_schema', element)

    # Get the parent of this Property/Parameter.
    if isinstance(element, data_types.ComplexDataType):
      parent = element
      data_type = element
    else:
      parent = element.schema
      data_type = element.data_type

    # If this is a reference then resolve it so we can check the details.
    data_type = getattr(data_type, 'referenced_schema', data_type)

    if not import_manager:
      if self._InnerModelClassesSupported():
        # If inner classes are supported find the top level parent to set
        # imports for.
        while parent.parent:
          parent = parent.parent
      import_manager = cpp_import_manager.CppImportManager.ForElement(parent)

    # For collections, we have to get the imports for the first non-collection
    # up the base_type chain.
    while isinstance(data_type, (data_types.ArrayDataType,
                                 data_types.MapDataType)):
      data_type = data_type._base_type  # pylint: disable=protected-access
      # TODO(user): This should be handled within the core.
      data_type = getattr(data_type, 'referenced_schema', data_type)

    # If we are a reference to a schema that does not resolve, do not fail.
    # Our kitchen sink test API does this. Well formed APIs should not.
    if not data_type:
      return

    json_type = data_type.json_type
    json_format = data_type.values.get('format')
    if json_type == 'object':
      if not data_type.parent:
        import_manager.AddDataType(data_type)
      return

    datatype_and_imports = (
        self.language_model.type_map.get((json_type, json_format)))
    if datatype_and_imports:
      import_definition = datatype_and_imports[1]
      # Import all required imports.
      for required_import in import_definition.imports:
        if required_import:
          import_manager.AddImport(required_import)
      # Set all template values, if specified.
      for template_value in import_definition.template_values:
        element.SetTemplateValue(template_value, True)
    elif data_type:
      if not data_type.parent:
        import_manager.AddDataType(data_type)
    return


class CppLanguageModel(language_model.LanguageModel):
  """A LanguageModel tuned for C++."""

  language = 'cpp'
  member_policy = language_model.NamingPolicy(
      language_model.LOWER_UNCAMEL_CASE, '{name}_', '_')
  getter_policy = language_model.NamingPolicy(
      language_model.LOWER_UNCAMEL_CASE, 'get_{name}', '_')
  setter_policy = language_model.NamingPolicy(
      language_model.LOWER_UNCAMEL_CASE, 'set_{name}', '_')
  has_policy = language_model.NamingPolicy(
      language_model.LOWER_UNCAMEL_CASE, 'has_{name}', '_')
  unset_policy = language_model.NamingPolicy(
      language_model.LOWER_UNCAMEL_CASE, 'clear_{name}', '_')

  parameter_name_policy = language_model.NamingPolicy(
      language_model.LOWER_UNCAMEL_CASE, '{name}', '_',
      conflict_policy=language_model.NamingPolicy(
          language_model.LOWER_UNCAMEL_CASE, '{name}__', '_'))

  # From http://en.cppreference.com/w/cpp/keyword
  _CPP_KEYWORDS = [
      'alignas', 'alignof', 'and', 'and_eq', 'asm', 'auto', 'bitand', 'bitor',
      'bool', 'break', 'case', 'catch', 'char', 'char16_t', 'char32_t',
      'class', 'compl', 'const', 'constexpr', 'const_cast', 'continue',
      'decltype', 'default', 'delete', 'do', 'double', 'dynamic_cast', 'else',
      'enum', 'explicit', 'export', 'extern', 'false', 'float', 'for',
      'friend', 'goto', 'if', 'inline', 'int', 'long', 'mutable', 'namespace',
      'new', 'noexcept', 'not', 'not_eq', 'nullptr', 'operator', 'or', 'or_eq',
      'private', 'protected', 'public', 'register', 'reinterpret_cast',
      'return', 'short', 'signed', 'sizeof', 'static', 'static_assert',
      'static_cast', 'struct', 'switch', 'template', 'this', 'thread_local',
      'throw', 'true', 'try', 'typedef', 'typeid', 'typename', 'union',
      'unsigned', 'using', 'virtual', 'void', 'volatile', 'wchar_t', 'while',
      'xor', 'xor_eq'
      ]

  _SPECIAL_CLASS_NAMES = [
      'JsonCppData', 'SerializableJson'
      ]

  # We can not create classes which match a C++ keyword or built in object
  # type or keyword. However since:
  #   * class names are case-sensitive
  #   * we'll always generate camel-case names (or least starting upper case)
  #   * no C++ keywords are uppercase
  # We only have to worry about classes built into the generator itself.
  # And really we do not need to even do that since they are in a different
  # namespace. But we will for convenience and since it's probably rare.
  RESERVED_CLASS_NAMES = _SPECIAL_CLASS_NAMES

  reserved_words = RESERVED_CLASS_NAMES + _CPP_KEYWORDS

  def __init__(self, options=None):
    super(CppLanguageModel, self).__init__(class_name_delimiter='::',
                                           module_name_delimiter='_')
    self.type_map = self._BuildTypeMap(options)

    self._SUPPORTED_TYPES['boolean'] = self._Boolean
    self._SUPPORTED_TYPES['integer'] = self._Int

  def _BuildTypeMap(self, options=None):
    """Builds the map of discovery document types to C++ types.

    Args:
      options: (dict) Code generator options.

    Returns:
      map of (discovery type, format) keys to (C++ type, import value)
      values where the import value is header file name to include.
    """
    self.global_namespace_ = ''
    self.client_namespace_ = 'client'
    builtin_type_h = None
    integral_type_h = '"googleapis/base/integral_types.h"'
    json_type_h = '"googleapis/client/data/jsoncpp_data.h"'
    # TODO(user): Implement open-source date.h
    date_time_h = '"googleapis/client/util/date_time.h"'
    date_h = date_time_h
    self.date_namespace_ = self.client_namespace_

    # Dictionary of json type and format to its corresponding data type and
    # import definition. The first import in the imports list is the primary
    # import.
    return {
        ('boolean', None): ('bool', ImportDefinition([builtin_type_h])),
        ('any', None): (self.client_namespace_ + '::JsonCppData',
                        ImportDefinition([json_type_h])),
        ('integer', None): ('int32', ImportDefinition([integral_type_h])),
        ('integer', 'int16'): ('int16', ImportDefinition([integral_type_h])),
        ('integer', 'uint16'): ('uint16', ImportDefinition([integral_type_h])),
        ('integer', 'int32'): ('int32', ImportDefinition([integral_type_h])),
        ('integer', 'uint32'): ('uint32', ImportDefinition([integral_type_h])),
        ('number', None): ('double', ImportDefinition([builtin_type_h])),
        ('number', 'double'): ('double', ImportDefinition([builtin_type_h])),
        ('number', 'float'): ('float', ImportDefinition([builtin_type_h])),
        ('object', None): (self.client_namespace_ + '::JsonCppData',
                           ImportDefinition([json_type_h])),
        ('string', None): ('string', ImportDefinition(['<string>'])),
        ('string', 'byte'): ('string', ImportDefinition(['<string>'])),
        ('string', 'date'): (self.date_namespace_ + '::Date',
                             ImportDefinition([date_h])),
        ('string', 'date-time'): (self.client_namespace_ + '::DateTime',
                                  ImportDefinition([date_time_h])),
        ('string', 'google-datetime'): ('string',
                                        ImportDefinition(['<string>'])),
        ('string', 'google-duration'): ('string',
                                        ImportDefinition(['<string>'])),
        ('string', 'google-fieldmask'): ('string',
                                         ImportDefinition(['<string>'])),
        ('string', 'int64'): ('int64', ImportDefinition([integral_type_h])),
        ('string', 'uint64'): ('uint64', ImportDefinition([integral_type_h])),
        }

  def _Boolean(self, data_value):
    """Convert provided boolean to language specific literal."""
    return unicode(bool(data_value.value)).lower()

  def _Int(self, data_value):
    """Convert provided int to language specific literal."""
    # Available types can be found in class variables
    code_types = {
        'int16': '%s',
        'uint16': '%s',
        'int32': '%s',
        'uint32': '%s',
        'int64': '%sL',
        'uint64': '%sL',
        }
    try:
      return code_types[data_value.code_type] % long(data_value.value)
    except KeyError:
      raise ValueError(
          ('Provided DataValue (%s) does not present an appropriate C++ '
           'annotated code type (%s).') %
          (data_value.value, data_value.code_type))

  def GetCodeTypeFromDictionary(self, def_dict):
    """Convert a json schema type to a suitable C++ type name.

    Overrides the default.

    Args:
      def_dict: (dict) A dictionary describing Json schema for this Property.
    Returns:
      A name suitable for use as a class in the generator's target language.
    """
    json_type = def_dict.get('type', 'string')
    json_format = def_dict.get('format')

    datatype_and_imports = self.type_map.get((json_type, json_format))
    if datatype_and_imports:
      # If there is an entry in the type format to datatype and imports
      # dictionary set it as the native format.
      native_format = datatype_and_imports[0]
    else:
      # Could not find it in the dictionary, set it to the json type.
      native_format = utilities.CamelCase(json_type)
    return native_format

  def ArrayOf(self, unused_variable, type_name):
    """Take a type name and return the syntax for an array of them.

    Overrides the default.

    Args:
      unused_variable: (DataType) the data we want an array of.
      type_name: (str) A type name.
    Returns:
      A language specific string meaning "an array of type_name".
    """
    # TODO(user): 20120817
    # Need to figure out whether the type is a primitive or not, then
    # return appropriate array type (data, string or primitive).
    return '%s::JsonCppArray<%s >' % (self.client_namespace_, type_name)

  def MapOf(self, unused_variable, type_name):
    """Take a type name and return the syntax for a map of String to them.

    Overrides the default.

    Args:
      unused_variable: (DataType) the data we want an array of.
      type_name: (str) A type name.
    Returns:
      A language specific string meaning "an array of type_name".
    """
    return '%s::JsonCppAssociativeArray<%s >' % (
        self.client_namespace_, type_name)

  def ToMemberName(self, s, the_api):
    """CamelCase a wire format name into a suitable C++ variable name."""
    camel_s = utilities.CamelCase(s)
    if s.lower() in CppLanguageModel.RESERVED_CLASS_NAMES:
      # Prepend the service name
      return '%s%s' % (the_api.values['name'], camel_s)
    return camel_s[0].lower() + camel_s[1:]

  # pylint: disable=unused-argument
  def ToSafeClassName(self, s, the_api, parent=None):
    """Convert a name to a suitable class name in C++.

    Subclasses should override as appropriate.

    Args:
      s: (str) A canonical name for data element. (Usually the API wire format)
      the_api: (Api) The API this element is part of. For use as a hint when the
        name cannot be used directly.
      parent: (schema) The parent we use to get a safe class name.
    Returns:
      A name suitable for use as an element in C++.
    """
    safe_class_name = utilities.CamelCase(s)
    if parent:
      for ancestor in parent.full_path:
        if ancestor.class_name == safe_class_name:
          safe_class_name = '%s%s' % (parent.class_name, safe_class_name)
    if safe_class_name.lower() in CppLanguageModel.RESERVED_CLASS_NAMES:
      # Prepend the service name
      safe_class_name = '%s%s' % (utilities.CamelCase(the_api.values['name']),
                                  utilities.CamelCase(s))
    return safe_class_name

  def DefaultContainerPathForOwner(self, module):
    """Overrides the default implementation."""
    result = ''
    if FLAGS['cpp_generator_add_owner_dir'].value:
      result = module.owner_name.lower().replace(' ', '_')
    return result


class CppApi(api.Api):
  """An Api with C++ annotations."""

  def __init__(self, discovery_doc, **unused_kwargs):
    super(CppApi, self).__init__(discovery_doc)

    # Build the module name from the API name with a trailing '_api'.
    # This guards against empty package names (which would be anonymous)
    # and also avoids some name clashes with built-in names or keywords.
    # We emit only lower-case files to be more C++-like.
    base = self.values['name']
    self.module.SetPath(base.lower() + '_api')

    self.SetTemplateValue('absl_include_path', 'absl')
    self.SetTemplateValue('base_include_path', 'googleapis/base')
    self.SetTemplateValue('strings_include_path', 'googleapis/strings')
    self.SetTemplateValue('client_include_path', 'googleapis/client')
    self.SetTemplateValue('global_namespace', 'googleapis')
    self.SetTemplateValue('client_namespace', 'client')
    self.SetTemplateValue('status_type', 'util::Status')

  def TopLevelModelClasses(self):
    """Return the models which are not children of another model.

    Adds an additional constraint on Api.TopLevelModelClasses to remove top
    level schemas which are arrays.

    Returns:
      list of Model.
    """
    return [m for m in self.ModelClasses()
            if not m.parent and not isinstance(m, data_types.ArrayDataType)]

  # pylint: disable=unused-argument
  def ToClassName(self, s, element, element_type=None):
    """Convert a discovery name to a suitable C++ class name.

    In C++, nested class names cannot share the same name as the outer class
    name.

    Overrides the default.

    Args:
      s: (str) A rosy name of data element.
      element: (object) The object we need a class name for.
      element_type: (str) The kind of object we need a class name for.
    Returns:
      A name suitable for use as a class in the generator's target language.
    """
    s = utilities.CamelCase(s)
    if s in CppLanguageModel.RESERVED_CLASS_NAMES:
      s = '%s%s' % (utilities.CamelCase(self.values['name']), s)

    decorator = ''
    if isinstance(element, api.Method):
      resource = element.parent
      while isinstance(resource, api.Resource):
        s = '%s_%s' % (resource.values['className'], s)
        resource = resource.parent
      decorator = 'Method'
    elif isinstance(element, api.Resource):
      decorator = 'Resource'
    elif isinstance(element, api.Api):
      if not s.endswith('Service'):
        decorator = 'Service'
    else:
      return s

    return '%s%s' % (s, decorator)



#!/usr/bin/python2.7
#
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for dart_generator."""

__author__ = 'sammccall@google.com (Sam McCall)'


from google.apputils import basetest
from googleapis.codegen import dart_generator


class DartGeneratorTest(basetest.TestCase):

  def testAnnotateParameter(self):
    """Test numeric enum names get turned into valid identifiers."""
    gen = dart_generator.DartGenerator({
        'name': 'dummy',
        'version': 'v1',
        'methods': {
            'frob': {
                'id': 'frob',
                'parameters': {
                    'normal': {
                        'type': 'string',
                        'enum': ['y', 'n'],
                        'enumDescriptions': ['Yes', 'No'],
                        'location': 'query',
                    },
                    'numeric': {
                        'type': 'string',
                        'enum': ['0', '1V'],
                        'enumDescriptions': ['Zero', 'One'],
                        'location': 'query',
                    },
                },
            },
        },
    })
    gen.AnnotateApiForLanguage(gen._api)

    frob = gen._api.GetTemplateValue('methods')[0]
    normal, numeric = frob.GetTemplateValue('parameters')
    # Might be the other order
    if normal.GetTemplateValue('wireName') == 'numeric':
      normal, numeric = numeric, normal

    normal_pairs = (normal.GetTemplateValue('enumType')
                    .GetTemplateValue('pairs'))
    self.assertEquals([('Y', 'y', 'Yes'), ('N', 'n', 'No')],
                      normal_pairs)

    numeric_pairs = (numeric.GetTemplateValue('enumType')
                     .GetTemplateValue('pairs'))
    self.assertEquals([('VALUE_0', '0', 'Zero'), ('VALUE_1V', '1V', 'One')],
                      numeric_pairs)

if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Template objects which represent data types.

This module contains objects which usable in templates and represent data type
idioms.
"""

__author__ = 'aiuto@google.com (Tony Aiuto)'

from googleapis.codegen import template_objects


class DataType(template_objects.CodeObject):
  """Template object which represents a data type.

  This is the base class for things which might be data type definitions, such
  as Schema objects derived from JSONSchema blocks or primitive types.
  """

  def __init__(self, def_dict, api, parent=None, language_model=None):
    """Construct a DataType.

    Args:
      def_dict: (dict) The discovery dictionary for this element.
      api: (Api) The Api instance which owns this element.
      parent: (CodeObject) The parent of this element.
      language_model: (LanguageModel) The language we are targetting.
        Dynamically defaults to the parent's language model.
    """
    super(DataType, self).__init__(def_dict, api, parent=parent,
                                   language_model=language_model)
    self._json_type = def_dict.get('type')
    # If the schema has an id, that is a good default class name for it.
    schema_id = def_dict.get('id')
    if schema_id:
      self.SetTemplateValue('className', schema_id)
    # Top level primitive and container classes end up as known schema objects,
    # because some languages will want to generate definitions for them. Thus
    # they need a module. ComplexDataType instances might set a different
    # module in their constructor. The check for the api is to facilitiate
    # unit tests, which sometimes create types without any API.
    # TODO(user): Do not set _module here. Let it be dyanmically found from
    # the parent when needed. But, there is currently a problem with
    # BuildSchemaDefinitions where the top level schemas do not have a parent
    # because we are calling 'self.DataTypeFromJson(def_dict, name)' without
    # the parent parameter. If we add parent there, all hell seems to break.
    # Fix that and remove this.
    if self.api and hasattr(self.api, 'model_module'):
      self._module = self.api.model_module

    # Set some specific annotations as template values.
    annotations = def_dict.get('annotations')
    if annotations:
      required = annotations.get('required')
      self.SetTemplateValue('required_for_methods', required)

  @property
  def json_type(self):
    """Expose the type element from the JSON Schema type definition."""
    return self._json_type

  @property
  def code_type(self):
    """Returns the string representing this datatype."""
    return self.values.get('codeType') or self.values.get('className')

  @property
  def safe_code_type(self):
    """Returns the safe code type representing this datatype."""
    return self.safeClassName or self.code_type

  @property
  def primitive_data_type(self):
    """Returns the language specific primitive representing this datatype."""
    return None

  @property
  def class_name(self):
    return self.GetTemplateValue('className')

  @property
  def safeClassName(self):  # pylint: disable=g-bad-name
    """Returns a language appropriate name for this object.

    This property should only be used during template expansion. It is computed
    once, using the LanguageModel in play, and then that value is cached.

    Returns:
      (str) a name for an instance of this object.
    """
    safe_class_name = self.GetTemplateValue('safe_class_name')
    if not safe_class_name:
      safe_class_name = self.values.get('wireName')
      if not safe_class_name:
        return None
      if self.language_model:
        safe_class_name = self.language_model.ToSafeClassName(
            safe_class_name, self._api, self._parent)
      self.SetTemplateValue('safeClassName', safe_class_name)
    return safe_class_name


class PrimitiveDataType(DataType):
  """DataType which represents a "built in" data type.

  Primitive types are those which are provided by the language or one of its
  packages, rather than those defined by the API.  A language specific
  generater should annotate PrimitiveDataType objects with a specific codeType
  before using them to generate code.
  """

  def __init__(self, def_dict, api, parent=None):
    """Construct a PrimitiveDataType.

    Args:
      def_dict: (dict) The discovery dictionary for this element.
      api: (Api) The Api instance which owns this element.
      parent: (TemplateObject) The parent of this object.
    """
    super(PrimitiveDataType, self).__init__(def_dict, api, parent=parent)
    self.SetTemplateValue('builtIn', True)
    self.SetTemplateValue('isScalar', True)

  @property
  def class_name(self):
    return self.code_type

  @property
  def fullClassName(self):  # pylint: disable=g-bad-name
    """Override the TemplateObject path chaining."""
    return self.code_type

  @property
  def code_type(self):
    """Returns the language specific type representing this datatype."""
    user_override = self.values.get('codeType')
    if user_override:
      return user_override
    if self.language_model:
      s = self.language_model.GetCodeTypeFromDictionary(self._def_dict)
      return s
    return self.values.get('type')

  @property
  def safe_code_type(self):
    """Returns the safe code type representing this datatype."""
    return self.code_type

  @property
  def primitive_data_type(self):
    """Returns the language specific type representing this datatype."""
    if self.language_model:
      s = self.language_model.GetPrimitiveTypeFromDictionary(self._def_dict)
      return s
    return None

  @property
  def json_format(self):
    """Expose the format element from the JSON Schema type definition."""
    return self.values.get('format')


class ComplexDataType(DataType):
  """A DataType which requires a definition: that is, not primitive.

  ComplexDataTypes are structured objects and containers of objects.
  """

  def __init__(self, default_name, def_dict, api, parent=None,
               language_model=None, wire_name=None):
    """Construct an ComplexDataType.

    Args:
      default_name: (str) The name to give this type if there is no 'id' in
        the default dict.
      def_dict: (dict) The discovery dictionary for this element.
      api: (Api) The Api instance which owns this element.
      parent: (CodeObject) The parent of this element.
      language_model: (LanguageModel) The language we are targetting.
        Dynamically defaults to the parent's language model.
      wire_name: (str) The identifier used in the wire protocol for this object.
    Raises:
      ValueError: if there is no identifing name for this object.
    """
    super(ComplexDataType, self).__init__(def_dict, api, parent=parent,
                                          language_model=language_model)
    name = def_dict.get('id') or default_name or wire_name
    if not name:
      raise ValueError(
          'Complex data types must have an id or be assigned a name: %s' %
          def_dict)
    self.SetTemplateValue('wireName', wire_name or name)

  @property
  def code_type(self):
    """Returns the string representing this datatype."""
    return self.values.get('codeType') or self.className

  @property
  def safe_code_type(self):
    """Returns the safe code type representing this datatype."""
    return self.safeClassName or self.code_type

  @property
  def class_name(self):
    return self.values.get('className')

  @property
  def className(self):  # pylint: disable=g-bad-name
    return self.class_name or self.safeClassName


class ContainerDataType(ComplexDataType):
  """Superclass for all DataTypes which represent containers."""

  def __init__(self, name, base_type, parent=None, wire_name=None):
    """Construct an ArrayDataType.

    Args:
      name: (str) The name to give this type if there is no 'id' in
        the default dict.
      base_type: (DataType) The DataType to represent an array of.
      parent: (TemplateObject) The parent of this object.
      wire_name: (str) The identifier used in the wire protocol for this object.
    """
    # Access to protected _language_model OK here. pylint: disable=protected-access
    super(ContainerDataType, self).__init__(
        name, {}, base_type.api, parent=parent,
        language_model=base_type._language_model,
        wire_name=wire_name)
    self._base_type = base_type
    self.SetTemplateValue('isContainer', True)
    self.SetTemplateValue('baseType', base_type)
    self.SetTemplateValue('builtIn', True)
    # TODO(user): This gets parenting right so language models propagate down.
    # We should invert the computation of code_type so we ask the language
    # model for code type of a primitive.
    if isinstance(base_type, PrimitiveDataType):
      self._base_type.SetParent(self)


class ArrayDataType(ContainerDataType):
  """DataType which represents a array of another DataType."""

  def __init__(self, name, base_type, parent=None, wire_name=None):
    """Construct an ArrayDataType.

    Args:
      name: (str) The name to give this type.
      base_type: (DataType) The DataType to represent an array of.
      parent: (TemplateObject) The parent of this object.
      wire_name: (str) The identifier used in the wire protocol for this object.
    """
    super(ArrayDataType, self).__init__(name, base_type, parent=parent,
                                        wire_name=wire_name)
    self._json_type = 'array'
    self.SetTemplateValue('arrayOf', base_type)

  @property
  def code_type(self):
    """Returns the string representing the datatype of this variable.

    Note: This may should only be called after the language model is set.

    Returns:
      (str) A printable representation of this data type.
    """
    return self.language_model.ArrayOf(self._base_type,
                                       self._base_type.code_type)

  @property
  def safe_code_type(self):
    return self.language_model.ArrayOf(self._base_type,
                                       self._base_type.safe_code_type)


class MapDataType(ContainerDataType):
  """DataType which represents a map of string to another DataType.

  This is the base class for things which might be data type definitions, such
  as Schema objects derived from JSONSchema blocks or primitive types.
  """

  def __init__(self, name, base_type, parent=None, wire_name=None):
    """Construct a MapDataType.

    Args:
      name: (str) The name to give this type.
      base_type: (DataType) The DataType to represent an map of string to.
      parent: (TemplateObject) The parent of this object.
      wire_name: (str) The identifier used in the wire protocol for this object.
    """
    super(MapDataType, self).__init__(name, base_type, parent=parent,
                                      wire_name=wire_name)
    self._json_type = 'map'
    self.SetTemplateValue('mapOf', base_type)

  @property
  def code_type(self):
    """Returns the string representing the datatype of this variable.

    Note: This may should only be called after the language model is set.

    Returns:
      (str) A printable representation of this data type.
    """
    return self.language_model.MapOf(self._base_type, self._base_type.code_type)

  @property
  def safe_code_type(self):
    """Returns the string representing the safe datatype of this variable.

    Note: This may should only be called after the language model is set.

    Returns:
      (str) A printable representation of this data type.
    """
    return self.language_model.MapOf(self._base_type,
                                     self._base_type.safe_code_type)


class SchemaReference(DataType):
  """DataType which represents a type alias to named schema.

  Provides a lazy reference to schema by name.
  """

  def __init__(self, referenced_schema_name, api):
    """Construct a SchemaReference.

    Args:
      referenced_schema_name: (str) The name of the schema we are referencing.
      api: (Api) The Api instance which owns this element.

    Returns:
      SchemaReference
    """
    super(SchemaReference, self).__init__({}, api)
    self._referenced_schema_name = referenced_schema_name
    self.SetTemplateValue('className', referenced_schema_name)
    self.SetTemplateValue('wireName', referenced_schema_name)
    self.SetTemplateValue('reference', True)

  # TODO(user): 20130227
  # I thought there was another way to do this, but I don't remember
  # right now. This feels like something we should do after parsing all
  # the schemas, so that we can resolve in one pass and not worry about
  # loading order.
  @property
  def referenced_schema(self):
    """Returns the concrete schema being referenced by this instance."""
    data_type = self
    while isinstance(data_type, SchemaReference):
      # pylint: disable=protected-access
      data_type = data_type.api.SchemaByName(data_type._referenced_schema_name)

    return data_type

  @property
  def values(self):
    """Forwards the 'values' property of this object to the referenced object.

    This enables GetTemplateValue called on a Ref to effectively return
    the value for the truly desired schema.

    This may be safely called at any time, but may not produce expected
    results until after the entire API has been parsed. In practice, this
    means that anything done during template expansion is fine.

    Returns:
      dict of values which can be used in template.
    """
    s = self.referenced_schema
    if s:
      return s.values
    return self._def_dict

  @property
  def code_type(self):
    """Returns the string representing the datatype of this variable."""
    s = self.referenced_schema
    if s:
      return s.code_type
    return self._def_dict.get('codeType') or self._def_dict.get('className')

  @property
  def safe_code_type(self):  # pylint: disable=g-bad-name
    if not self.referenced_schema:
      return '<bad $ref>'
    return self.referenced_schema.safe_code_type

  @property
  def parent(self):
    """Returns the parent of the schema I reference."""
    return self.referenced_schema.parent

  @property
  def module(self):
    """Returns the module of the schema I reference."""
    return self.referenced_schema.module

  def __str__(self):
    return '<SchemaReference to %s>' % self.code_type


class Void(PrimitiveDataType):
  """DataType which represents a 'void'.

  Some API methods have no response. To provide some consistency in assigning
  a responseType to these methods, we use the Void data type. When it is
  referenced in a template, it forwards requests for it's code_type to a
  langauge model specific emitter.
  """

  def __init__(self, api):
    """Construct a Void.

    Args:
      api: (Api) The Api instance which owns this element. This is used for
        a parent chain so that we can pick up the language model at template
        generation time.

    Returns:
      Void
    """
    super(Void, self).__init__({}, api, parent=api)
    self.SetTemplateValue('isVoid', True)

  @property
  def code_type(self):
    """Returns the string representing the datatype of this variable."""
    if self.language_model:
      return self.language_model.CodeTypeForVoid()
    return 'void'


class Enum(PrimitiveDataType):
  """The definition of an Enum.

  Example enum in discovery.
    "enum": [
        "@comments",
        "@consumption",
        "@liked",
        "@public",
        "@self"
       ],
    "enumDescriptions": [
        "Limit to activities commented on by the user.",
        "Limit to activities to be consumed by the user.",
        "Limit to activities liked by the user.",
        "Limit to public activities posted by the user.",
        "Limit to activities posted by the user."
       ]
  """

  def __init__(self, def_dict, api, wire_name, values, descriptions, parent):
    """Create an enum.

    Args:
      def_dict: (dict) The discovery dictionary for this element.
      api: (Api) The Api which owns this Property
      wire_name: (str) The identifier used in the wire protocol for this enum.
      values: ([str]) List of possible values. If not provided, use the 'enum'
          element from def_dict.
      descriptions: ([str]) List of value descriptions. If not provided, use
          the 'enumDescriptions' element from def_dict.
      parent: (Method) The object owning this enum.
    """
    super(Enum, self).__init__(def_dict, api, parent=parent)
    name = def_dict.get('id') or wire_name
    self.ValidateName(name)
    self.SetTemplateValue('wireName', name)
    self.SetTemplateValue('className',
                          api.ToClassName(name, self, element_type='enum'))
    if values is None:
      values = def_dict.get('enum')
    if descriptions is None:
      descriptions = def_dict.get('enumDescriptions') or []

    self._elements = []
    for i in range(len(values)):
      v = values[i]
      # Sometimes the description list is too short.
      d = descriptions[i] if (i < len(descriptions)) else None
      self._elements.append(
          template_objects.Constant(v, description=d, parent=self))
    self.SetTemplateValue(
        'elements', self._elements,
        meaning='The individual possible values of an Enum data type.')

    # TODO(user): Migrate users away from the enum pairs to 'elements' and
    # delete the rest of this method.
    def FixName(name):
      name = name[0].isdigit() and 'VALUE_' + name or name.lstrip('@')
      return name.upper().replace('-', '_')
    names = [FixName(s) for s in values]
    def FixDescription(desc):
      return self.ValidateAndSanitizeComment(self.StripHTML(desc))
    pairs = zip(names, values, map(FixDescription, descriptions))
    self.SetTemplateValue('pairs', pairs)

  @property
  def enum_name(self):
    return self.language_model.ApplyPolicy('enum', self,
                                           self.values['wireName'])


def CreatePrimitiveDataType(def_dict, api, wire_name, parent=None):
  """Creates a PrimitiveDataType from a JSON dictionary.

  Creates a primitive built in type or an enum for a blob of json.

  Args:
    def_dict: (dict) The discovery dictionary for this element.
    api: (Api) The Api instance which owns this element.
    wire_name: (str) The identifier used in the wire protocol for this object.
    parent: (TemplateObject) The parent of this object.
  Returns:
    (PrimitiveDataType) A data type.
  """
  if def_dict.get('enum'):
    return Enum(def_dict,
                api,
                wire_name,
                None,
                None,
                parent)
  return PrimitiveDataType(def_dict, api, parent)



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for java_generator."""

__author__ = 'aiuto@google.com (Tony Aiuto)'

from google.apputils import basetest
from googleapis.codegen import data_types
from googleapis.codegen import data_value
from googleapis.codegen import java_generator


class JavaApiTest(basetest.TestCase):

  def testToClassName(self):
    """Test creating safe class names from object names."""
    # 'parameters': {}
    api = java_generator.JavaApi({
        'name': 'dummy',
        'version': 'v1',
        'resources': {
            'foo': {
                'methods': {
                    'bar': {
                        'id': 'bar',
                        }
                    }
                }
            }
        })
    self.assertEquals('Foo', api.ToClassName('foo', api))
    self.assertEquals('DummyClass', api.ToClassName('class', None))
    self.assertEquals('DummyDefault', api.ToClassName('default', None))
    self.assertEquals('DummyImport', api.ToClassName('import', None))
    self.assertEquals('DummyObject', api.ToClassName('object', None))
    self.assertEquals('DummyString', api.ToClassName('string', None))
    self.assertEquals('DummyTrue', api.ToClassName('true', None))
    self.assertEquals('dummy', api.values['name'])
    self.assertEquals('Dummy', api._class_name)

    # Test the renaming of the object when it matches the API name.
    self.assertEquals('Dummy', api.ToClassName('dummy', api))
    foo = api._resources[0]
    self.assertEquals('DummyOperations',
                      api.ToClassName('dummy', foo, element_type='resource'))
    bar = foo._methods[0]
    self.assertEquals('DummyOperation',
                      api.ToClassName('dummy', bar, element_type='method'))

  def testToClassNameWithCanonical(self):
    api = java_generator.JavaApi({
        'name': 'dummy',
        'canonicalName': 'Dummy Service',
        'version': 'v1',
        'resources': {}
        })
    self.assertEquals('dummy', api.values['name'])
    self.assertEquals('DummyService', api._class_name)
    self.assertEquals('DummyServiceClass', api.ToClassName('class', None))

  def testGetCodeTypeFromDictionary(self):
    """Test mapping of JSON schema types to Java class names."""
    language_model = java_generator.JavaLanguageModel()
    test_cases = [
        ['java.lang.String', {'type': 'string', 'format': 'byte'}],
        ['com.google.api.client.util.DateTime',
         {'type': 'string', 'format': 'date-time'}],
        ['java.lang.Double', {'type': 'number', 'format': 'double'}],
        ['java.lang.Float', {'type': 'number', 'format': 'float'}],
        ['java.lang.Short', {'type': 'integer', 'format': 'int16'}],
        ['java.lang.Integer', {'type': 'integer', 'format': 'int32'}],
        ['java.lang.Long', {'type': 'string', 'format': 'int64'}],
        ['java.lang.Object', {'type': 'any'}],
        ['java.lang.Boolean', {'type': 'boolean'}],
        ['java.lang.String', {'type': 'string'}],
        ['java.lang.Long', {'type': 'integer', 'format': 'uint32'}],
        ['java.math.BigInteger', {'type': 'string', 'format': 'uint64'}],
    ]
    for test_case in test_cases:
      self.assertEquals(
          test_case[0],
          language_model.GetCodeTypeFromDictionary(test_case[1]))

  def testGetPrimitiveTypeFromDictionary(self):
    """Test mapping of JSON schema types to Java class names."""
    language_model = java_generator.JavaLanguageModel()
    test_cases = [
        ['double', {'type': 'number', 'format': 'double'}],
        ['float', {'type': 'number', 'format': 'float'}],
        ['short', {'type': 'integer', 'format': 'int16'}],
        ['int', {'type': 'integer', 'format': 'int32'}],
        ['long', {'type': 'string', 'format': 'int64'}],
        ['boolean', {'type': 'boolean'}],
        ['long', {'type': 'integer', 'format': 'uint32'}],
        [None, {'type': 'string'}],
        [None, {'type': 'string', 'format': 'byte'}],
        [None, {'type': 'string', 'format': 'date-time'}],
        [None, {'type': 'string', 'format': 'uint64'}],
        [None, {'type': 'anything_else', 'format': 'uint64'}],
    ]

    for test_case in test_cases:
      self.assertEquals(
          test_case[0],
          language_model.GetPrimitiveTypeFromDictionary(test_case[1]))


class JavaGeneratorTest(basetest.TestCase):

  def testImportsForArray(self):
    """Test if we get the right imports for an array.

    The goal is to see that an array of a primative type which requires an
    import really works.
    """
    gen = java_generator.BaseJavaGenerator({
        'name': 'dummy',
        'version': 'v1',
        'resources': {},
        'schemas': {
            'Bar': {
                'id': 'Bar',
                'type': 'object',
                'properties': {
                    'p1': {
                        'type': 'array',
                        'items': {
                            'type': 'string',
                            'format': 'uint64'
                            }
                        },
                    'p2': {
                        'type': 'array',
                        'items': {
                            'type': 'array',
                            'items': {
                                'type': 'string',
                                'format': 'date-time'
                                }
                            }
                        }
                    }
                }
            }
        })
    gen.AnnotateApiForLanguage(gen.api)
    found_big_integer = False
    found_date_time = False
    for schema in gen._api._schemas.values():
      import_manager = schema.values.get('importManager')
      for import_list in import_manager.ImportLists():
        for import_def in import_list:
          print import_def
          if import_def.find('BigInteger'):
            found_big_integer = True
          if import_def.find('DateTime'):
            found_date_time = True
    self.assertTrue(found_big_integer)
    self.assertTrue(found_date_time)


class JavaLanguageModelTest(basetest.TestCase):
  """Tests for features implemented in the language model."""


  def testDefaultPath(self):
    """Test the package path generation."""
    def MakeGen(host):
      gen = java_generator.BaseJavaGenerator({
          'name': 'fake',
          'version': 'v1',
          'rootUrl': 'https://%s/' % host,
          'servicePath': 'fake/v1',
          'ownerDomain': host,
          })
      gen.AnnotateApiForLanguage(gen.api)
      return gen

    gen = MakeGen('google.com')
    self.assertEquals('com/google/fake', gen.api.module.path)
    self.assertEquals('com/google/fake/model', gen.api.model_module.path)

    gen = MakeGen('not-google.com')
    self.assertEquals('com/not_google/fake', gen.api.module.path)
    self.assertEquals('com.not_google.fake', gen.api.module.name)
    gen = MakeGen('my-custom_app.appspot.com')
    self.assertEquals('com/appspot/my_custom_app/fake', gen.api.module.path)

  def testDefaultPathWithPackagePathAndCanonicalName(self):
    """Test the package path generation."""
    def MakeGen():
      api_def = {
          'name': 'fake',
          'version': 'v1',
          'rootUrl': 'https://www.googleapis.com/',
          'servicePath': 'fake/v1',
          'ownerDomain': 'google.com',
          'packagePath': 'my/path',
          'canonicalName': 'Canonical Name'
      }
      gen = java_generator.BaseJavaGenerator(
          api_def, options={'google_api': True})
      gen.AnnotateApiForLanguage(gen.api)
      return gen

    gen = MakeGen()
    self.assertEquals('com/google/my/path/canonicalname', gen.api.module.path)
    self.assertEquals('com/google/my/path/canonicalname/model', gen.api.model_module.path)

  def testAllowedCharacters(self):
    # make sure $ is allowed in a name and that @ is not

    model = java_generator.JavaLanguageModel()
    self.assertEquals('$ref',
                      model.TransformString(None, '$ref', model.member_policy))
    self.assertEquals('set$ref',
                      model.TransformString(None, '$ref', model.setter_policy))
    self.assertEquals('getId',
                      model.TransformString(None, '@id', model.getter_policy))


class JavaLanguageModelDataValueTest(basetest.TestCase):
  """Tests for DataValue integration."""

  def setUp(self):
    self.language_model = java_generator.JavaLanguageModel()

  def _CreateDataValue(self, value, val_type):
    def_dict = {
        'className': 'Foo',
        'type': val_type,
        }
    prototype = data_types.DataType(
        def_dict, None, language_model=self.language_model)
    dv = data_value.DataValue(value, prototype)
    return dv

  def testRenderBoolean(self):
    dv = self._CreateDataValue(True, 'boolean')
    render_method = self.language_model._SUPPORTED_TYPES['boolean']
    self.assertEqual('true', render_method(dv))

    dv.SetValue(False)
    self.assertEqual('false', render_method(dv))

  def testRenderInteger(self):
    dv = self._CreateDataValue(42, 'integer')
    render_method = self.language_model._SUPPORTED_TYPES['integer']
    self.assertRaises(ValueError, render_method, dv)

    dv.SetTemplateValue('codeType', 'java.lang.Long')
    self.assertEqual('42L', render_method(dv))


class Java14LanguageModelTest(basetest.TestCase):

  def setUp(self):
    self.language_model = java_generator.JavaLanguageModel()

  def _CreateDataValue(self, value, val_type):
    def_dict = {
        'className': 'Foo',
        'type': val_type,
        }
    prototype = data_types.DataType(
        def_dict, None, language_model=self.language_model)
    dv = data_value.DataValue(value, prototype)
    return dv

  def testRenderBoolean(self):
    dv = self._CreateDataValue(True, 'boolean')
    render_method = self.language_model._SUPPORTED_TYPES['boolean']
    self.assertEqual('true', render_method(dv))

    dv.SetValue(False)
    self.assertEqual('false', render_method(dv))


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.

"""ObjC library generator.

This module generates a an Objective-C client library for a Discovery based API.
"""

__author__ = 'aiuto@google.com (Tony Aiuto)'

import re
from googleapis.codegen import api
from googleapis.codegen import api_library_generator
from googleapis.codegen import data_types
from googleapis.codegen import language_model
from googleapis.codegen import objc_import_manager
from googleapis.codegen import utilities


class ObjCGenerator(api_library_generator.ApiLibraryGenerator):
  """The ObjC code generator."""

  _FORMAT_TO_TYPE_COMMENT = {
      'int16': 'shortValue',
      'int32': 'intValue',
      'int64': 'longLongValue',
      'uint16': 'unsignedShortValue',
      'uint32': 'unsignedIntValue',
      'uint64': 'unsignedLongLongValue',
      'float': 'floatValue',
      'double': 'doubleValue',
      'byte': 'GTLBase64 can encode/decode (probably web-safe format)'
      }

  # Common parameters are listed at the service level; they apply to all
  # methods. However, there are some that come back from discovery that don't
  # make sense at the method level for the GTL library, so we filter those out.
  _COMMON_PARAMS_TO_SKIP = [
      'alt',  # GTL only supports JSON
      'key',  # GTL handles the API key on the service, per method is need in
              # the ObjectiveC use case.
      'oauth_token',  # GTL handles this on the service.
      'prettyPrint',  # No needed, GTL Logging pretty prints for the developer.
      'userIp',  # Not needed for client software, only needed for servers
                 # making request on behalf of lots of users.
      'quotaUser',  # Another form of userIp.
      ]

  # Regex to match properties needing the NS_RETURNS_NOT_RETAINED directive.
  _NOT_RETAINED = re.compile('^(new|copy|mutableCopy)')

  def __init__(self, discovery, options=None):
    super(ObjCGenerator, self).__init__(
        ObjCApi,
        discovery,
        language='objc',
        language_model=ObjCLanguageModel(options),
        options=options)

  def AnnotateSchema(self, unused_api, schema):
    schema.SetTemplateValue('superClass', 'GTLObject')

  def AnnotateApi(self, api_object):
    api_object.SetTemplateValue('enums', [])
    name = api_object.values['canonicalName']
    api_name_policy = language_model.NamingPolicy(
        language_model.UPPER_CAMEL_CASE)
    camel_case_name = self._language_model.TransformString(
        api_object, name, api_name_policy)
    api_object.SetTemplateValue('serviceName',
                                'GTLService%s' % (camel_case_name))
    rpc_url = api_object.GetTemplateValue('rpcUrl')
    if rpc_url:
      rpc_url += '?prettyPrint=false'
      api_object.SetTemplateValue('rpcUrl', rpc_url)
      # Only generate query class for RPC discovery documents, not REST.
      api_object.SetTemplateValue('query', 'GTLQuery%s' % (camel_case_name))

    api_object.SetTemplateValue(
        'parameters',
        [param for param in api_object.GetTemplateValue('parameters')
         if param.GetTemplateValue('wireName') not in
         self._COMMON_PARAMS_TO_SKIP])

    super(ObjCGenerator, self).AnnotateApi(api_object)

    # Collect all method parameters for combined query class.
    # The query class has properties for all method parameters on the API, with
    # a static method for each API method to generate the appropriate query
    # object. There are likely to be naming clashes where two or more API
    # methods have parameters of the same name but different types. When this
    # happens, we use the 'id' type instead.
    method_parameters = {}
    for method in api_object.GetTemplateValue('methods'):
      for parameter in method.parameters:
        # Check for duplicate
        existing_parameter = method_parameters.get(parameter.codeName)
        if not existing_parameter:
          method_parameters[parameter.codeName] = parameter
        elif existing_parameter.codeType != parameter.codeType:
          # Duplicate exists with mismatched type; synthesise new parameter
          # using 'id' type instead.
          merged_param = api.Parameter(
              api_object,
              parameter.GetTemplateValue('wireName'),
              {
                  'description':
                  '"%s" has different types for some query methods; see the '
                  'documentation for the right type for each query method.'
                  % (parameter.codeName),
                  'wireName': parameter['wireName'],
                  'codeName': parameter.codeName,
              },
              existing_parameter.parent)
          merged_param.SetTemplateValue('codeType', 'id')
          merged_param.SetTemplateValue('attributes', 'nonatomic, retain')
          merged_param.SetTemplateValue('noPointer', True)
          method_parameters[parameter.codeName] = merged_param
    api_object.SetTemplateValue(
        'methodParameters',
        [method_parameters[key] for key in sorted(method_parameters)])

    # Also build a sorted list of all parameters, both common and per-method
    all_parameters = method_parameters.copy()
    for parameter in api_object.GetTemplateValue('parameters'):
      all_parameters[parameter.codeName] = parameter
    api_object.SetTemplateValue(
        'allParameters',
        [all_parameters[key] for key in sorted(all_parameters)])

  def AnnotateProperty(self, api_object, prop, schema):
    """Annotate a Property with ObjC specific elements."""
    if (prop.codeName == 'items' and
        isinstance(prop.data_type, data_types.ArrayDataType)):
      # Set superclass if schema has an items array.
      schema.SetTemplateValue('superClass', 'GTLCollectionObject')

    self.AnnotateParameterOrProperty(prop, schema, api_object)

    # It appears that data_types.Enum is not actually used
    # TODO(user): Once enum properties are handled properly with
    # data_types.Enum, fix this workaround.
    enum_values = prop.data_type.values.get('enum')
    if enum_values and not isinstance(schema, data_types.SchemaReference):
      self.AnnotateEnum(api_object, prop, schema, enum_values)

  def AnnotateEnum(self, api_object, prop, schema, enum_values):
    descriptions = prop.data_type.values.get('enumDescriptions')
    class_name = schema.GetTemplateValue('className')
    property_name = prop.codeName[0].upper() + prop.codeName[1:]
    enum_elements = [
        {
            'value': enum_values[i],
            'description': descriptions and descriptions[i],
            'constantName': self.language_model.TransformString(
                prop,
                enum_values[i].lower(),
                self.language_model.constant_policy)
        }
        for i in range(len(enum_values))]
    enum = {
        'comment': '%s - %s' % (class_name, property_name),
        'className': class_name,
        'propertyName': property_name,
        'elements': enum_elements,
    }
    enums = api_object.GetTemplateValue('enums')
    enums.append(enum)
    api_object.SetTemplateValue('enums', sorted(enums))

  def AnnotateParameter(self, method, parameter):
    self.AnnotateParameterOrProperty(parameter, method.api, method.api)

  def AnnotateParameterOrProperty(self, prop, schema, api_object):
    """Common annotations that apply to both parameters and properties."""
    # Property attributes depend on type
    if prop.data_type.code_type == 'NSString':
      prop.SetTemplateValue('attributes', 'nonatomic, copy')
    else:
      prop.SetTemplateValue('attributes', 'nonatomic, retain')

    # Properties starting with certain strings will cause a clang warning about
    # naming conventions unless we add a directive to stop it.
    if self._NOT_RETAINED.match(prop.codeName):
      prop.SetTemplateValue('clangDirective', ' NS_RETURNS_NOT_RETAINED')

    if isinstance(prop.data_type, data_types.ArrayDataType):
      # Add comment showing type of array elements
      prop.SetTemplateValue('typeComment',
                            'of %s' % (prop.data_type['baseType'].code_type))
      prop.SetTemplateValue('elementCodeType',
                            prop.data_type['baseType'].code_type)
      # If there are any array properties then we need to generate an
      # arrayPropertyToClassMap method.
      schema.SetTemplateValue('arrayPropertyToClassMap', True)
      # If it is an array of enum values then we need to handle the enum type.
      enum_values = prop.data_type['baseType'].values.get('enum')
      if enum_values:
        self.AnnotateEnum(api_object, prop, schema, enum_values)
    elif prop.data_type.code_type in ['NSNumber', 'NSString']:
      if prop.data_type.json_format:
        prop.SetTemplateValue(
            'typeComment',
            self._FORMAT_TO_TYPE_COMMENT.get(prop.data_type.json_format))
    elif prop.GetTemplateValue('type'):
      prop.SetTemplateValue('typeComment', prop.GetTemplateValue('type'))

    # We need to generate a propertyToJSONKeyMap if at least one property has a
    # different code name to wire name.
    if prop.codeName != prop.GetTemplateValue('wireName'):
      schema.SetTemplateValue('propertyToJSONKeyMap', True)

    self.AnnotateType(prop.data_type, schema)

  def AnnotateMethod(self, api_object, method, resource):
    super(ObjCGenerator, self).AnnotateMethod(api_object, method, resource)
    name = method['wireName']
    method.SetTemplateValue(
        'queryCodeName',
        self.language_model.TransformString(
            method,
            name[name.find('.'):],
            self.language_model.query_method_policy))

  def AnnotateType(self, data_type, schema):
    # Import type of property if it is a class
    if (isinstance(data_type, data_types.SchemaReference) or
        isinstance(data_type, api.Schema)):
      import_manager = objc_import_manager.ObjCImportManager.ForElement(schema)
      import_manager.AddDataType(data_type)
    elif isinstance(data_type, data_types.ArrayDataType):
      self.AnnotateType(data_type.GetTemplateValue('baseType'), schema)


class ObjCLanguageModel(language_model.LanguageModel):
  """A LanguageModel for Objective-C."""

  array_of_policy = language_model.NamingPolicy(format_string='NSArray')
  map_of_policy = language_model.NamingPolicy(format_string='NSDictionary')
  constant_policy = language_model.NamingPolicy(
      case_transform=language_model.UPPER_CAMEL_CASE,
      format_string='k{schema.className}_{variable_name}_{name}')
  query_method_policy = language_model.NamingPolicy(
      case_transform=language_model.UPPER_CAMEL_CASE,
      format_string='queryFor{name}')

  _SCHEMA_TYPE_TO_OBJC_TYPE = {
      ('any', None): 'String',
      ('boolean', None): 'NSNumber',
      ('integer', None): 'NSNumber',
      ('integer', 'int16'): 'NSNumber',
      ('integer', 'uint16'): 'NSNumber',
      ('integer', 'int32'): 'NSNumber',
      ('integer', 'uint32'): 'NSNumber',
      ('number', None): 'NSNumber',
      ('number', 'double'): 'NSNumber',
      ('number', 'float'): 'NSNumber',
      ('object', None): 'GTLObject',
      ('string', None): 'NSString',
      ('string', 'byte'): 'NSString',
      ('string', 'date'): 'NSString',
      ('string', 'date-time'): 'GTLDateTime',
      ('string', 'int64'): 'NSNumber',
      ('string', 'uint64'): 'NSNumber',
      }

  # http://www.binpress.com/tutorial/objective-c-reserved-keywords/43
  _OBJC_KEYWORDS = [
      'asm', 'auto', 'break', 'case', 'char', 'const', 'continue', 'default',
      'do', 'double', 'else', 'enum', 'extern', 'float', 'for', 'goto',
      'if', 'inline', 'int', 'long', 'register', 'restrict', 'return', 'short',
      'signed', 'sizeof', 'static', 'struct', 'switch', 'typedef', 'union',
      'unsigned', 'void', 'volatile', 'while', '_Bool', '_Complex',
      '_Imaginary',
      ]
  # C/C++ keywords (including C99 and C++ 0x11)
  _CPP_KEYWORDS = [
      'and', 'alignas', 'alignof', 'bitand', 'bitor', 'bool', 'catch', 'compl',
      'constexpr', 'decltype', 'default', 'delete', 'explicit', 'export',
      'false', 'friend', 'mutable', 'namespace', 'new', 'noexcept', 'not',
      'nullptr', 'operator', 'or', 'private', 'protected', 'public', 'template',
      'this', 'throw', 'true', 'try', 'typeid', 'typename', 'using', 'virtual',
      'xor', 'restrict',
      ]
  _NAMES_TO_AVOID = [
      'bool', 'bycopy', 'byref', 'class', 'id', 'imp', 'in', 'inout', 'nil',
      'no', 'null', 'object', 'oneway', 'out', 'protocol', 'sel', 'self',
      'string', 'super', 'yes', 'atomic', 'nonatomic', 'retain',
      ]

  # We can not create classes which match a ObjC keyword or built in object
  # type.
  RESERVED_CLASS_NAMES = _OBJC_KEYWORDS + _CPP_KEYWORDS + _NAMES_TO_AVOID

  # We can not create data members which are in NSObject or GTLObject.
  RESERVED_MEMBER_NAMES = RESERVED_CLASS_NAMES + [
      # NSObject methods
      'description', 'debugDescription', 'finalize', 'hash', 'dealloc', 'init',
      'superclass', 'retain', 'release', 'autorelease', 'retainCount', 'zone',
      'isProxy', 'classForCoder',
      # Foundation protocol methods
      'copy', 'copyWithZone', 'mutableCopy', 'mutableCopyWithZone',
      # GTLObject methods
      'JSON', 'JSONString', 'surrogates', 'userProperties',
      'additionalJSONKeys', 'additionalProperties', 'userData',
      'fieldsDescription',
      ]
  reserved_words = RESERVED_MEMBER_NAMES

  def __init__(self, options=None):
    super(ObjCLanguageModel, self).__init__()
    self._options = options

  def GetCodeTypeFromDictionary(self, def_dict):
    """Convert a json primitive type to a suitable ObjC type name.

    Overrides the default.

    Args:
      def_dict: (dict) A dictionary describing Json schema for this Property.
    Returns:
      A name suitable for use as a class in the generator's target language.
    """
    json_type = def_dict.get('type', 'string')
    json_format = def_dict.get('format')
    native_format = self._SCHEMA_TYPE_TO_OBJC_TYPE.get((json_type, json_format),
                                                       json_type)
    return native_format

  def ToMemberName(self, s, unused_api):
    """Convert a wire format name into a suitable ObjC variable name."""
    if s.lower() == 'id':  # Special case for id
      return 'identifier'
    camel_s = utilities.CamelCase(s)
    ret = camel_s[0].lower() + camel_s[1:]
    if s.lower() in ObjCLanguageModel.RESERVED_MEMBER_NAMES:
      ret = '%sProperty' % ret
    return ret

  def DefaultContainerPathForOwner(self, module):
    return '/'.join(utilities.ReversedDomainComponents(module.owner_domain))


class ObjCApi(api.Api):
  """An Api with Objective-C annotations."""

  def __init__(self, discovery_doc, **unused_kwargs):
    super(ObjCApi, self).__init__(discovery_doc)

  # pylint: disable=unused-argument
  def ToClassName(self, s, element, element_type=None):
    """Convert a discovery name to a suitable ObjC class name.

    Overrides the default.

    Args:
      s: (str) A rosy name of data element.
      element: (object) The object we need a class name for.
      element_type: (str) The kind of element to name.
    Returns:
      A name suitable for use as a class in the generator's target language.
    """
    if ((s.lower() in ObjCLanguageModel.RESERVED_CLASS_NAMES)
        or isinstance(element, api.Schema)):
      # Prepend the service name
      name = self.values['canonicalName']
      s = '%s%s' % (
          utilities.CamelCase(name).replace(' ', ''),
          utilities.CamelCase(s))
    else:
      s = utilities.CamelCase(s).replace(' ', '')
    if not s.startswith('GTL'):
      return 'GTL' + s
    return s



#!/usr/bin/python2.7
"""Tests for csharp generator."""

from google.apputils import basetest
from googleapis.codegen import csharp_generator


class NestedClassNameTest(basetest.TestCase):

  NESTED_API = {
      'name': 'nestor',
      'version': 'v1',
      'resources': {},
      'schemas': {
          'A': {
              'id': 'A',
              'type': 'object',
              'properties': {
                  'B': {
                      'id': 'B',
                      'type': 'object',
                      'properties': {
                          'count': {
                              'type': 'integer',
                              'format': 'int32'
                              }
                          }
                      }
                  }
              },
          'C': {
              'id': 'C',
              'type': 'array',
              'items': {
                  '$ref': 'A'
                  }
              },
          'D': {
              'id': 'D',
              'type': 'array',
              'items': {
                  'type': 'object',
                  'properties': {
                      'foo': {
                          'type': 'string'
                          }
                      }
                  }
              },
          'E': {
              'id': 'E',
              'type': 'object',
              'properties': {
                  'F': {
                      'type': 'array',
                      'items': {
                          'type': 'object',
                          'properties': {
                              'aproperty': {
                                  'type': 'string'
                                  }
                              }
                          }
                      }
                  }
              }
          }
      }

  COMPUTE_EXTRACT = {
      'name': 'computeextract',
      'version': 'v1',
      'resources': {},
      'schemas': {
          'MachineType': {
              'id': 'MachineType',
              'type': 'object',
              'properties': {
                  'ephemeralDisks': {
                      'type': 'array',
                      'items': {
                          'type': 'object',
                          'properties': {
                              'diskGb': {
                                  'type': 'integer',
                                  'format': 'int32'
                                  }
                              }
                          }
                      }
                  }
              }
          }
      }

  FREEBASE_EXTRACT = {
      'name': 'freebase',
      'version': 'v1',
      'resources': {},
      'schemas': {
          'TopicStatslinkcount': {
              'id': 'TopicStatslinkcount',
              'type': 'object',
              'properties': {
                  'type': {
                      'type': 'string',
                      'default': 'custom'
                      },
                  'values': {
                      'type': 'array',
                      'items': {
                          'type': 'object',
                          'properties': {
                              'count': {
                                  'type': 'integer',
                                  'format': 'int32'
                                  },
                              'id': {
                                  'type': 'string'
                                  },
                              'values': {
                                  'type': 'array',
                                  'items': {
                                      'type': 'object',
                                      'properties': {
                                          'count': {
                                              'type': 'integer',
                                              'format': 'int32'
                                              },
                                          'id': {
                                              'type': 'string'
                                              },
                                          'values': {
                                              'type': 'array',
                                              'items': {
                                                  'type': 'object',
                                                  'properties': {
                                                      'count': {
                                                          'type': 'integer',
                                                          'format': 'int32'
                                                          },
                                                      'id': {
                                                          'type': 'string'
                                                          }
                                                      }
                                                  }
                                              }
                                          }
                                      }
                                  }
                              }
                          }
                      }
                  }
              }
          }
      }

  def testNestedClassName(self):
    gen = csharp_generator.CSharpGenerator(discovery=self.NESTED_API)
    gen.AnnotateApiForLanguage(gen.api)
    self.assertEquals({'A', 'A.B', 'C', 'D', 'DItems', 'E', 'E.F'},
                      set(gen.api._schemas))
    b_schema = gen.api._schemas['A.B']
    c_schema = gen.api._schemas['C']
    self.assertEquals('A.B', b_schema.packageRelativeClassName)
    # TODO(user): This is the current behavior.  We could change it for
    # csharp by annotating schemas and setting codeType to
    # packageRelativeClassName, but that will uncover other bugs -- due to the
    # many ways of getting at a class name (codeType, code_type, safeCodeType,
    # safe_code_type, class_name, className, safe...) there is inconsistency in
    # the code base, and we'll end up prepending parent names in some cases more
    # than once.
    self.assertEquals('B', b_schema.code_type)
    self.assertEquals('System.Collections.Generic.IList<A>', c_schema.code_type)
    d_schema = gen.api._schemas['D']
    self.assertEquals('System.Collections.Generic.IList<DItems>',
                      d_schema.code_type)

  def testAbsurdlyNestedButRealClassNames(self):
    gen = csharp_generator.CSharpGenerator(discovery=self.FREEBASE_EXTRACT)
    gen.AnnotateApiForLanguage(gen.api)
    schema_names = sorted(s.class_name.split('.')[-1]
                          for s in gen.api._schemas.itervalues())
    expected = sorted(['TopicStatslinkcount',
                       'ValuesData',  # nested 1
                       'ValuesDataSchema',  # nested 2
                       'ValuesData',  # nested 3
                      ])
    self.assertEquals(expected, schema_names)

  def testNestedArray(self):
    gen = csharp_generator.CSharpGenerator(discovery=self.COMPUTE_EXTRACT)
    gen.AnnotateApiForLanguage(gen.api)
    self.assertEquals({'MachineType', 'MachineType.ephemeralDisks'},
                      set(gen.api._schemas))
    # The array type in this case is not available in the schema list; we have
    # to spelunk into the properties to get at it.
    machine_type_schema = gen.api._schemas['MachineType']
    # While properties in discovery are dictionaries, for some reason
    # the dictionary is overridden and replaced by a dict in the object model.
    disk_array = [p for p in machine_type_schema['properties']
                  if p['wireName'] == 'ephemeralDisks'][0]
    self.assertEquals(
        'System.Collections.Generic.IList<MachineType.EphemeralDisksData>',
        disk_array.code_type)
    ephemeral_disk_schema = gen.api._schemas['MachineType.ephemeralDisks']
    self.assertEquals('EphemeralDisksData', ephemeral_disk_schema.safeClassName)


class PackagePathTest(basetest.TestCase):

  def testPackagePath(self):
    discovery = {
        'name': 'foo',
        'version': 'v1',
        'packagePath': 'nougat',
        'schemas': {},
        'resources': {},
        }
    gen = csharp_generator.CSharpGenerator(discovery)
    gen.AnnotateApiForLanguage(gen.api)
    api = gen.api
    self.assertEquals('Google.Apis.Nougat.Foo.v1', api.module.name)
    self.assertEquals('Google.Apis.Nougat.Foo.v1.Data', api.model_module.name)


class VersionWithDashTest(basetest.TestCase):

  def testVersionWithDash(self):
    discovery = {
        'name': 'foo',
        'version': 'v1-dash',
        'schemas': {},
        'resources': {},
        }
    gen = csharp_generator.CSharpGenerator(discovery)
    gen.AnnotateApiForLanguage(gen.api)
    api = gen.api
    self.assertEquals('Google.Apis.Foo.v1dash', api.module.name)
    self.assertEquals('Google.Apis.Foo.v1dash.Data', api.model_module.name)


class MemberNameTest(basetest.TestCase):

  def testMemberNameConflictWithParentClass(self):
    api = {
        'name': 'groups',
        'version': 'v1',
        'schemas': {
            'Groups': {
                'id': 'Groups',
                'type': 'object',
                'properties': {
                    'groups': {
                        'type': 'string'
                        }
                    }
                }
            }
        }
    gen = csharp_generator.CSharpGenerator(discovery=api)
    gen.AnnotateApiForLanguage(gen.api)
    groups_schema = gen.api._schemas['Groups']
    groups_prop = groups_schema['properties'][0]
    self.assertEquals('GroupsValue', groups_prop.memberName)

  def testMemberNameConflictWithParentClassWithUnderscores(self):
    api = {
        'name': 'groups',
        'version': 'v1',
        'schemas': {
            'GroupsThing': {
                'id': 'groups_thing',
                'type': 'object',
                'properties': {
                    'groups_thing': {
                        'type': 'string'
                        }
                    }
                }
            }
        }
    gen = csharp_generator.CSharpGenerator(discovery=api)
    gen.AnnotateApiForLanguage(gen.api)
    groups_schema = gen.api._schemas['GroupsThing']
    groups_prop = groups_schema['properties'][0]
    self.assertEquals('GroupsThingValue', groups_prop.memberName)


class MapTest(basetest.TestCase):

  def testMapNameIsConsistent(self):
    api = {
        'name': 'discovery',
        'version': 'v1',
        'schemas': {
            'RestDescription': {
                'id': 'RestDescription',
                'type': 'object',
                'properties': {
                    'auth': {
                        'type': 'object',
                        'properties': {
                            'oauth2': {
                                'type': 'object',
                                'properties': {
                                    'scopes': {
                                        'type': 'object',
                                        'additionalProperties': {
                                            'type': 'object',
                                            'properties': {
                                                'description': {
                                                    'type': 'string',
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    gen = csharp_generator.CSharpGenerator(discovery=api)
    gen.AnnotateApiForLanguage(gen.api)
    schemas = gen.api._schemas
    oauth2_schema = schemas['RestDescription.auth.oauth2']
    scopes_schema = schemas['RestDescription.auth.oauth2.scopesElement']
    child_schema = oauth2_schema.children[0]
    self.assertEquals(scopes_schema, child_schema)
    self.assertEquals('ScopesDataElement', scopes_schema.safeClassName)


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Base library generator.

This module holds the base classes used for all code generators.
"""

__author__ = 'aiuto@google.com (Tony Aiuto)'

import datetime
import os
import re
import StringIO
import time
import zipfile


from googleapis.codegen.django_helpers import DjangoRenderTemplate
from googleapis.codegen.language_model import LanguageModel
from googleapis.codegen.template_objects import UseableInTemplates
# Has to be after django_helpers pylint: disable=g-bad-import-order
from googleapis.codegen import template_helpers
from googleapis.codegen.filesys import files

# This block is static information about the generator which will get passed
# into templates.
_GENERATOR_INFORMATION = {
    'name': 'google-apis-code-generator',
    'version': '1.6.0',
    'buildDate': '2019-01-29',
    }

# app.yaml and other names that app engine refuses to open.
# TODO(user) Remove once templates are stored in BlobStore.
_SPECIAL_FILENAMES = ['app_yaml']


class TemplateGenerator(object):
  """Base class for walking a template tree to generate output files.

  This class provides methods for processing template trees to produce output
  trees.
  * Provides a common base dictionary of variables for use in templates.
  * Callers can augment that with their own dictionary of variables.
  * Callers can provide a set of replacements to be made to file paths in
    the template tree
  """

  def __init__(self, language_model=None, options=None):
    self._tool_info = ToolInformation()
    self._options = options or dict()
    self._template_dir = os.path.dirname(__file__)
    self._surface_features = {}
    self._language_model = language_model or LanguageModel()

  @property
  def language_model(self):
    return self._language_model

  def IncludeFileTree(self, path_to_tree, package):
    """Walk a file tree and copy files directly into an output package.

    Walks a file tree relative to the target language, copying all files
    found into an output package.

    Args:
      path_to_tree: (str) path relative to the language template directory
      package: (LibraryPackage) output package.
    """
    top_of_tree = os.path.join(self._template_dir, path_to_tree)
    # Walk tree for jar files to directly include
    for path in files.IterFiles(top_of_tree):
      relative_path = path[len(top_of_tree) + 1:]
      package.IncludeFile(path, relative_path)

  def PathToTemplate(self, template_name):
    """Returns the full path to a template."""
    return os.path.join(self._template_dir, template_name)

  def RenderTemplate(self, template_path, context_dict=None):
    """Render a template.

    Renders a template with the standard dictionary of bindings.

    Args:
      template_path: (str) Full path to a template.
      context_dict: (dict) A dictionary to augment the standard template
        dictionary.
    Returns:
      (str) The fully rendered template string.
    """
    variables_dict = {
        'tool': self._tool_info,  # Information about the build tool
        'options': self._options,  # Options for this invocation
        'template_dir': self._template_dir,  # path to the template tree
        'features': self._surface_features,  # sub language options
        'language_model': self._language_model
        }
    if context_dict:
      variables_dict.update(context_dict)
    return DjangoRenderTemplate(template_path, variables_dict)

  def RenderTemplateToFile(self, template_path, context_dict, package,
                           output_path):
    """Render a template as a file in the output package.

    Args:
      template_path: (str) Full path to a template.
      context_dict: (dict) A dictionary to augment the standard template
        dictionary.
      package: (LibraryPackage) output package.
      output_path: (str) file path in the package.

    Returns:
      None
    """
    output_dir, file_name = os.path.split(output_path)

    def WriteFileInPackage(path, content):
      """Writes content to a path in our current package writer."""
      if isinstance(content, unicode):
        content = content.encode('utf-8', errors='ignore')
      out = package.StartFile(os.path.join(output_dir, path))
      out.write(content)
      package.EndFile()

    try:
      context_dict[template_helpers.FILE_WRITER] = WriteFileInPackage
      content = self.RenderTemplate(template_path, context_dict)
      WriteFileInPackage(file_name, content)
    except template_helpers.Halt:
      pass
    del context_dict[template_helpers.FILE_WRITER]

  def WalkTemplateTree(self, path_to_tree, path_replacements, list_replacements,
                       variables, package, file_filter=None):
    """Walk a file tree and copy files or process templates.

    Walks a file tree to write on output package, running all files ending in
    ".tmpl" through the template renderer, and directly copying all the other
    files. While doing so, the caller may provide for some transformations of
    the file path:
    1. They can pass in a dictionary of string literals to replacements which
       are applied directly to the file path. E.g. '___package___' might be
       replaced by a path snippet of the form 'com/google/tasks'
    2. They can pass a dictionary of strings literals which, if found in the
       file path trigger the evaluation of thats template against a list of
       CodeObjects, with a distinct output file being generated for each object.
       E.g. The ApiLibraryGenerator uses the pattern  '___models_className___'
       to generate a set of files, each named by its className member.

    Args:
      path_to_tree: (str) path relative to the language template directory.
      path_replacements: (dict) dict holding elements which should be replaced
        if found in a path.
      list_replacements: (dict) dict holding elements which should be replaced
        by many files when found in a path. The keys of the dict are strings
        to be found in a path. The values are a tuple of
           (name_to_bind, [list of code objects])
        where name_to_bind is a variable name which will be bound to each
        successive code object during template evaluation. See the
        GenerateListOfFiles method for more details about name expansion.
      variables: (dict) The dictionary of variable replacements to pass to the
         templates.
      package: (LibraryPackage) output package.
      file_filter: (func) method to allow the caller to filter files included
         by name. The method is called with 2 arguments, the template file path
         and the path after all path replacements are done.
    """

    def ExpandZipFile(path):
      """Expand a zip file found in the template tree.

      Args:
        path: Path to file, relative to top of template tree.
      """
      full_path = os.path.join(self._template_dir, path)
      zip_slurp = files.GetFileContents(full_path)
      archive = zipfile.ZipFile(StringIO.StringIO(zip_slurp), 'r')
      for info in archive.infolist():
        package.WriteDataAsFile(
            archive.read(info.filename),
            os.path.join(relative_path, info.filename))

    top_of_tree = os.path.normpath(
        os.path.join(self._template_dir, path_to_tree))
    # Walk tree for jar files to directly include
    variables.update({'template_dir': top_of_tree})
    for path in files.IterFiles(top_of_tree):
      root, file_name = os.path.split(path)
      template_path = file_name
      relative_path = root[len(top_of_tree) + 1:]

      # Perform the replacements on the path and file name
      for path_item, replacement in path_replacements.iteritems():
        relative_path = relative_path.replace(path_item, replacement)
      for path_item, replacement in path_replacements.iteritems():
        file_name = file_name.replace(path_item, replacement)
      full_template_path = os.path.join(relative_path, template_path)

      for path_item, call_info in list_replacements.iteritems():
        if file_name.find(path_item) >= 0:
          self.GenerateListOfFiles(path_item, call_info, path, relative_path,
                                   file_name, variables, package,
                                   file_filter=file_filter)

      if file_name.startswith('___unzip___'):
        # TODO(user) Doesn't account for changes to relative_path above.
        ExpandZipFile(path)
        continue
      if file_name.startswith('_'):
        continue
      if file_name.endswith('.tmpl'):
        name_in_zip = file_name[:-5]  # strip '.tmpl'
        if name_in_zip in _SPECIAL_FILENAMES:
          name_in_zip = name_in_zip.replace('_', '.')
        full_output_path = os.path.join(relative_path, name_in_zip)
        if file_filter and not file_filter(full_template_path,
                                           full_output_path):
          continue
        self.RenderTemplateToFile(path, variables, package, full_output_path)
      else:
        full_output_path = os.path.join(relative_path, file_name)
        if file_filter and not file_filter(full_template_path,
                                           full_output_path):
          continue
        package.IncludeFile(path, full_output_path)

  def GeneratePackage(self, package_writer):
    """Generate the package.

    Args:
      package_writer: (LibraryPackage) output package
    """
    # COV_NF_START
    raise NotImplementedError(
        'GeneratePackage must be implemented by all subclasses')
    # COV_NF_END

  def DefaultGeneratePackage(self, package_writer, path_replacements,
                             variables):
    """Default operations to generate the package.

    Do all the default operations for generating a package.
    1. Walk the template tree to generate the source.
    2. Optionally copy in dependencies

    This is a utility method intended for subclasses of TemplateGenerator, so
    that they may implement the bulk of GeneratePackage by calling this.

    Args:
      package_writer: (LibraryPackage) output package.
      path_replacements: (dict) dict holding elements which should be replaced
         if found in a path.
      variables: (dict) The dictionary of variable replacements to pass to the
         templates.
    """
    self.WalkTemplateTree('templates', path_replacements, {}, variables,
                          package_writer)

  def SetFeatures(self, surface_features):
    """Sets the dict to be used for the 'features' variable in templates."""
    self._surface_features = surface_features

  @property
  def features(self):
    return self._surface_features

  @features.setter
  def features(self, surface_features):
    self._surface_features = surface_features

  @property
  def language_version(self):
    if self._surface_features:
      return self._surface_features.get('releaseVersion')

  def SetTemplateDir(self, template_dir):
    """Sets the template directory tree to use for WalkTemplateTree.

    Args:
      template_dir: (str) Path to template tree. If it is an absolute path it
        will be used directly. If relative, it is taken relative to the source
        tree.
    """
    if template_dir.startswith('/'):
      self._template_dir = template_dir
    else:
      self._template_dir = os.path.join(self._template_dir, template_dir)

  def GenerateListOfFiles(self, path_prefix, call_info, template_path,
                          relative_path, template_file_name, variables,
                          package, file_filter=None):
    """Generate many output files from a template.

    This method blends together a list of CodeObjects (from call_info) with
    the template_file_name to produce an output file for each of the elements
    in the list. The names for each file are derived from a template variable
    of each element.

    Args:
      path_prefix: (str) The piece of path which triggers the replacement.
      call_info: (list) ['name to bind', [list of CodeObjects]]
      template_path: (str) The path of the template file.
      relative_path: (str) The relative path of the output file in the package.
      template_file_name: (str) the file name of the template for this list.
        The file name must contain the form '{path_prefix}{variable_name}___'
        (without the braces). The pair is replaced by the value of variable_name
        from each successive element of the call list.
      variables: (dict) The dictionary of variable replacements to pass to the
         templates.
      package: (LibraryWriter) The output package stream to write to.
      file_filter: (func) See WalkTemplateTree for a description.

    Raises:
      ValueError: If the template_file_name does not match the call_info data.
    """
    path_and_var_regex = r'%s([a-z][A-Za-z]*)___' % path_prefix
    match_obj = re.compile(path_and_var_regex).match(template_file_name)
    if not match_obj:
      raise ValueError(
          'file names which match path item for GenerateListOfFiles must'
          ' contain a variable for substitution. E.g. "___models_codeName___"')
    variable_name = match_obj.group(1)
    file_name_piece_to_replace = path_prefix + variable_name + '___'
    for element in call_info[1]:
      file_name = template_file_name.replace(
          file_name_piece_to_replace, element.values[variable_name])
      name_in_zip = file_name[:-5]  # strip '.tmpl'
      if file_filter and not file_filter(None, name_in_zip):
        continue
      d = dict(variables)
      d[call_info[0]] = element
      self.RenderTemplateToFile(
          template_path, d, package, os.path.join(relative_path, name_in_zip))


class ToolInformation(UseableInTemplates):
  """Defines information about this generator tool itself."""

  def __init__(self):
    super(ToolInformation, self).__init__(_GENERATOR_INFORMATION)
    now = datetime.datetime.utcnow()
    self.SetTemplateValue('runDate',
                          '%4d-%02d-%02d' % (now.year, now.month, now.day))
    self.SetTemplateValue(
        'runTime',
        '%02d:%02d:%02d UTC' % (now.hour, now.minute, now.second))



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


"""Command line tool to run the Google API library generator.
Usage:
$ PYTHONPATH=$(/bin/pwd)/src \
  $(/bin/pwd)/src/googleapis/codegen/generate_library.py \
    --api_name=plus --api_version=v1 --output_dir=plus_lib
"""

__author__ = 'aiuto@google.com (Tony Aiuto)'

import collections
import json
import logging
import os


import httplib2

from google.apputils import app
import gflags as flags
from google.apputils import resources
from googleapis.codegen import generator_lookup
from googleapis.codegen.filesys import package_writer_foundry
from googleapis.codegen.targets import Targets

FLAGS = flags.FLAGS


flags.DEFINE_string(
    'api_name',
    None,
    'API to generate for.  E.g. "plus".')
flags.DEFINE_string(
    'api_version',
    None,
    'version of "api_name" to generate for.  E.g. "v1".')
flags.DEFINE_string(
    'discovery_server',
    'www.googleapis.com',
    'The discovery server to use for loading "api_name"')
flags.DEFINE_string(
    'discovery_version',
    'v1',
    'The discovery version to use for loading "api_name"')
flags.DEFINE_boolean(
    'include_timestamp',
    False,
    'Adds a timestamp to the generated source files.')
flags.DEFINE_string(
    'input',
    None,
    'A discovery document captured from a discovery service.')
flags.DEFINE_enum(
    'language',
    'java',
    generator_lookup.SupportedLanguages(),
    'Target language for the generated library')
flags.DEFINE_string(
    'language_variant',
    'default',
    'which variant of "language" to generate for. E.g. "stable" vs. "head".')
flags.DEFINE_string(
    'monolithic_source_name',
    None,
    ('If non-empty then combine all the sources to the single file'
     ' <monolithic_source_name>.{extension} and all the headers to a single'
     ' include file. Otherwise use the standard output file naming and produce'
     ' a source file for the overall API and one for each schema. This option'
     ' is only supported for C++ and Objective-C.'))
flags.DEFINE_string(
    'output_dir',
    None,
    'A path to a directory where the generated files will be created.')
flags.DEFINE_string(
    'output_file',
    None,
    'An output file path to contain the archive for the generated library.'
    ' The contents of the file are determined by the output_format parameter')
flags.DEFINE_enum(
    'output_format',
    'zip',
    ['zip', 'tgz', 'tar', 'txt'],
    'What format to use for --output_file.')
flags.DEFINE_enum(
    'output_type',
    'plain',
    ['plain', 'full'],
    'What kind of output to make.'
    ' plain=just the source,'
    ' full=turn on all the optional parts (useful for testing the generator).'
    )
flags.DEFINE_string(
    'package_path',
    None,
    'Use an alternate path for the generated code. This must be a file path'
    ' using "/" as a separator, not "."'
    )
flags.DEFINE_boolean(
    'reparent_methods_using_id',
    False,
    'If true, move methods to parent resource if the ID matches parent path')
flags.DEFINE_bool('version_package', False, 'Put API version in package paths')
flags.DEFINE_bool('verbose', False, 'Enable verbose logging')

flags.declare_key_flag('api_name')
flags.declare_key_flag('api_version')
flags.declare_key_flag('include_timestamp')
flags.declare_key_flag('input')
flags.declare_key_flag('language')
flags.declare_key_flag('language_variant')
flags.declare_key_flag('monolithic_source_name')
flags.declare_key_flag('output_dir')
flags.declare_key_flag('output_file')
flags.declare_key_flag('output_format')
flags.declare_key_flag('output_type')
flags.declare_key_flag('package_path')
flags.declare_key_flag('reparent_methods_using_id')
flags.declare_key_flag('version_package')


def main(unused_argv):
  if not (FLAGS.api_name or FLAGS.input):
    raise app.UsageError('You must specify one of --api_name or --input')
  if FLAGS.api_name and FLAGS.input:
    raise app.UsageError(
        'You can only specify one of --api_name or --input')
  if not (FLAGS.output_dir or FLAGS.output_file):
    raise app.UsageError(
        'You must specify one of --output_dir or --output_file')
  if FLAGS.output_dir and FLAGS.output_file:
    raise app.UsageError(
        'You can only specify one of --output_dir or --output_file')

  if FLAGS.verbose:
    logging.basicConfig(level=logging.DEBUG)


  # Get the discovery document
  if FLAGS.api_name:
    if not FLAGS.api_version:
      raise app.UsageError('You must specify --api_version with --api_name')
    content = GetApiDiscovery(FLAGS.api_name, FLAGS.api_version)
  else:
    f = open(FLAGS.input)
    content = f.read()
    f.close()
  discovery_doc = json.loads(content, object_pairs_hook=collections.OrderedDict)


  package_writer = package_writer_foundry.GetPackageWriter(
      output_dir=FLAGS.output_dir, output_file=FLAGS.output_file,
      output_format=FLAGS.output_format)

  Generate(discovery_doc=discovery_doc,
           package_writer=package_writer,
           include_timestamp=FLAGS.include_timestamp,
           version_package=FLAGS.version_package,
           package_path=FLAGS.package_path,
           output_type=FLAGS.output_type,
           language=FLAGS.language,
           language_variant=FLAGS.language_variant)
  return 0


def Generate(discovery_doc, package_writer,
             include_timestamp=False,
             version_package=False,
             package_path=None,
             output_type='plain',
             language='java',
             language_variant='default',
             callback=None):
  """Generate a library package from discovery and options."""
  options = {
      # Include other files needed to compile (e.g. base jar files)
      'include_dependencies': False,
      # Include the timestamp in the generated library
      'include_timestamp': include_timestamp,
      # Put API version in the package
      'version_package': version_package,
      # Custom package name
      'package_path': package_path,
      }
  if FLAGS.monolithic_source_name:
    options['useSingleSourceFile'] = True
  if output_type == 'full':
    options['include_dependencies'] = True
  if FLAGS.reparent_methods_using_id:
    discovery_doc['reparentMethodsUsingId'] = True

  # determine language version from language variant.
  language_variations = Targets().VariationsForLanguage(language)
  if not language_variations:
    raise app.UsageError('Language %s missing from '
                         'apiserving/libgen/gen/targets.json' %
                         language)
  features = language_variations.GetFeatures(language_variant)
  if not features:
    raise app.UsageError('Unsupported language variant: '
                         '%s/%s/features.json is missing' %
                         language, language_variant)
  try:
    generator_class = generator_lookup.GetGeneratorByLanguage(
        features.get('generator', language))
  except ValueError:
    raise app.UsageError('Unsupported language: %s' % language)

  generator = generator_class(discovery_doc, options=options)
  if FLAGS.monolithic_source_name:
    generator.api.SetTemplateValue('monolithicSourceName',
                                   FLAGS.monolithic_source_name)
  generator.SetTemplateDir(features.template_dir)
  generator.SetFeatures(features)
  generator.GeneratePackage(package_writer)
  package_writer.DoneWritingArchive()
  if callback:
    callback(discovery_doc=discovery_doc,
             package_writer=package_writer,
             include_timestamp=include_timestamp,
             version_package=version_package,
             package_path=package_path,
             output_type=output_type,
             language=language,
             language_variant=language_variant)


def GetApiDiscovery(api_name, api_version):
  """Get a discovery doc from the discovery server."""
  api_path = 'apis/%s/%s/rest' % (api_name, api_version)

  discovery_url = 'https://%s/discovery/%s/%s' % (
      FLAGS.discovery_server, FLAGS.discovery_version, api_path)
  http = httplib2.Http()
  _, content = http.request(discovery_url)
  discovery_doc = json.loads(content)
  error = discovery_doc.get('error')
  if error:
    raise app.Error(error)
  return content


if __name__ == '__main__':
  app.run()



#!/usr/bin/python2.7
# Copyright 2012 Google Inc. All Rights Reserved.

"""Tests for language_model.DocumentingLanguageModel."""

__author__ = 'aiuto@google.com (Tony Aiuto)'

from google.apputils import basetest
from googleapis.codegen import language_model


class DocumentingLanguageModelTest(basetest.TestCase):

  def testDocumentingLanguageModel(self):
    dlm = language_model.DocumentingLanguageModel()
    self.assertEquals('Array<foo>', dlm.ArrayOf(None, 'foo'))
    self.assertEquals('Map<string, foo>', dlm.MapOf(None, 'foo'))
    self.assertEquals('foo', dlm.GetCodeTypeFromDictionary({'type': 'foo'}))
    self.assertEquals('foo (int)', dlm.GetCodeTypeFromDictionary({
        'type': 'foo', 'format': 'int'}))


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2012 Google Inc. All Rights Reserved.

"""Support for simple JSON templates.

A JSON template is a dictionary of JSON data in which string values
may be simple templates in string.Template format (i.e.,
$dollarSignEscaping).  By default, the template is expanded against
its own data, optionally updated with additional context.
"""

import json
from string import Template
import sys

__author__ = 'smulloni@google.com (Jacob Smullyan)'


def ExpandJsonTemplate(json_data, extra_context=None, use_self=True):
  """Recursively template-expand a json dict against itself or other context.

  The context for string expansion is the json dict itself by default, updated
  by extra_context, if supplied.

  Args:
    json_data: (dict) A JSON object where string values may be templates.
    extra_context: (dict) Additional context for template expansion.
    use_self: (bool) Whether to expand the template against itself, or only use
        extra_context.

  Returns:
    A dict where string template values have been expanded against
    the context.
  """
  if use_self:
    context = dict(json_data)
  else:
    context = {}
  if extra_context:
    context.update(extra_context)

  def RecursiveExpand(obj):
    if isinstance(obj, list):
      return [RecursiveExpand(x) for x in obj]
    elif isinstance(obj, dict):
      return dict((k, RecursiveExpand(v)) for k, v in obj.iteritems())
    elif isinstance(obj, (str, unicode)):
      return Template(obj).safe_substitute(context)
    else:
      return obj

  return RecursiveExpand(json_data)


if __name__ == '__main__':
  if len(sys.argv) > 1:
    json_in = open(sys.argv[1])
  else:
    json_in = sys.stdin
  data = json.load(json_in)
  expanded = ExpandJsonTemplate(data)
  json.dump(expanded, sys.stdout, indent=2)



#!/usr/bin/python2.7
"""Tests for json_with_comments.py."""

from google.apputils import basetest

from googleapis.codegen.utilities import json_with_comments


class JsonWithCommentsTest(basetest.TestCase):

  SOME_JSON_WITH_COMMENTS = """
    # Garlic and sapphires in the mud
    # Clot the bedded axle-tree.
    {"author": "Timmy",
     "books": [
       # Diet classic
       "The Waist Band"
     ]
    }
    # The End
    """

  JSON_CONTENT = {'author': 'Timmy',
                  'books': ['The Waist Band']}

  def testLineNumbering(self):
    stripped = json_with_comments._StripComments(self.SOME_JSON_WITH_COMMENTS)
    # The stripped version should have the same number of line breaks as the
    # original.
    num_lines = self.SOME_JSON_WITH_COMMENTS.count('\n')
    self.assertEquals(num_lines, stripped.count('\n'))

  def testLoads(self):
    data = json_with_comments.Loads(self.SOME_JSON_WITH_COMMENTS)
    self.assertEquals(self.JSON_CONTENT, data)


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for java_import_manager."""

__author__ = 'rmistry@google.com (Ravi Mistry)'

from google.apputils import basetest
from googleapis.codegen.java_import_manager import JavaImportManager


class MockSchema(object):
  """Mock schema used in place of real schema objects."""

  def __init__(self):
    self._template_values_dict = {}

  def SetTemplateValue(self, template_name, template_value):
    self._template_values_dict[template_name] = template_value

  def GetTemplateValue(self, template_name):
    return self._template_values_dict.get(template_name)


class JavaImportManagerTest(basetest.TestCase):

  def setUp(self):
    super(JavaImportManagerTest, self).setUp()
    self.mock_schema = MockSchema()
    self.import_manager = JavaImportManager(self.mock_schema)

  def testAddImportAndCommit(self):
    # Try to add a java.lang import. It should not add an import but still
    # return True since it is added by default.
    self.assertTrue(self.import_manager.AddImport('java.lang.String'))
    self.assertFalse(self.import_manager.google_imports)
    self.assertFalse(self.import_manager.other_imports)
    self.assertFalse(self.import_manager.java_imports)

    # Add a com.google import.
    com_google_import = 'com.google.api.client.util.DateTime'
    self.assertTrue(self.import_manager.AddImport(com_google_import))

    # Add a java.util import.
    java_util_import = 'java.util.Map'
    self.assertTrue(self.import_manager.AddImport(java_util_import))

    # Add a random thing
    other_namespace_import = 'org.foo.bar'
    self.import_manager.AddImport(other_namespace_import)

    # Assert the contents of google, other and java imports.
    self.assertEquals([com_google_import], self.import_manager.google_imports)
    self.assertEquals([other_namespace_import],
                      self.import_manager.other_imports)
    self.assertEquals([java_util_import], self.import_manager.java_imports)

    # Assert the contents of class_name_to_qualified_name map.
    self.assertEquals(java_util_import,
                      self.import_manager._class_name_to_qualified_name['Map'])
    self.assertEquals(
        com_google_import,
        self.import_manager._class_name_to_qualified_name['DateTime'])

    # Assert that commit import works.
    expected_import_list = [
        [com_google_import],
        [java_util_import],
        [other_namespace_import]]
    self.assertEquals(
        expected_import_list,
        self.mock_schema.GetTemplateValue('importManager').ImportLists())

  def testAddImportConflictingNames(self):
    datetime1_import = 'com.google.api.client.util.DateTime'
    datetime2_import = 'com.test.testing.util.DateTime'
    self.assertTrue(self.import_manager.AddImport(datetime1_import))
    # Adding the 2nd datetime import should return a false since the name
    # is already imported.
    self.assertFalse(self.import_manager.AddImport(datetime2_import))
    self.assertEquals(
        datetime1_import,
        self.import_manager._class_name_to_qualified_name['DateTime'])

  def testGetClassName(self):
    self.assertEqual('Boolean',
                     self.import_manager.GetClassName('java.lang.Boolean'))


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2012 Google Inc. All Rights Reserved.
#

"""Common generator index utilities."""

__author__ = 'akesling@google.com (Alex Kesling)'


from googleapis.codegen import cpp_generator
from googleapis.codegen import csharp_generator
from googleapis.codegen import dart_generator
from googleapis.codegen import gwt_generator
from googleapis.codegen import java_generator
from googleapis.codegen import objc_generator
from googleapis.codegen import php_generator
from googleapis.codegen import python_generator
from googleapis.codegen import sample_generator


# Multiple generators per language are possible, as is the case with
# Java below. Template trees can specify a specific generator in their
# features.json file (with the "generator" attribute); this will refer
# to a key in these dictionaries.  If a template tree does not
# include this specification, the language name is used as a key.
_GENERATORS_BY_LANGUAGE = {
    'cpp': cpp_generator.CppGenerator,
    'csharp': csharp_generator.CSharpGenerator,
    'dart': dart_generator.DartGenerator,
    'gwt': gwt_generator.GwtGenerator,
    'java': java_generator.Java14Generator,
    # TODO(user): Consider cleaning up all the old java/*/features.json files
    # to use 'java' as the generator. It seems right to keep the ability to
    # have different version stamp styles so we can support old code
    # generators, but this feature is of dubious value.
    'java1_15': java_generator.Java14Generator,
    'objc': objc_generator.ObjCGenerator,
    'php': php_generator.PHPGenerator,
    'python': python_generator.PythonGenerator,
    'sample': sample_generator.SampleGenerator,
    }


def GetGeneratorByLanguage(language_or_generator):
  """Return the appropriate generator for this language.

  Args:
    language_or_generator: (str) the language for which to return a generator,
        or the name of a specific generator.

  Raises:
    ValueError: If provided language isn't supported.

  Returns:
    The appropriate code generator object (which may be None).
  """

  try:
    return _GENERATORS_BY_LANGUAGE[language_or_generator]
  except KeyError:
    raise ValueError('Unsupported language: %s' % language_or_generator)


def SupportedLanguages():
  """Return the list of languages we support.

  Returns:
    list(str)
  """
  return sorted(_GENERATORS_BY_LANGUAGE)



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Base classes for objects which are usable in templates.

This module contains the base classes for objects which can be used directly
in template expansion.
"""

__author__ = 'aiuto@google.com (Tony Aiuto)'

import copy

from googleapis.codegen import utilities
from googleapis.codegen.django_helpers import MarkSafe
from googleapis.codegen.utilities import html_stripper
from googleapis.codegen.utilities import name_validator


class UseableInTemplates(object):
  """Base class for any object usable in templates.

  The important feature is that they function as dicts, so that their properties
  can be referenced from django templates.
  """

  def __init__(self, def_dict):
    """Construct a UseableInTemplates object.

    Args:
      def_dict: The discovery dictionary for this element. All the values in it
          are exposed to the template expander.
    """
    # TODO(user): Do we really need both of these?  Can def_dict
    # simply be a deep copy? Or can we store mutations separately and
    # thus not change the underlying dictionary?
    self._def_dict = dict(def_dict)
    self._raw_def_dict = dict(copy.deepcopy(def_dict))

  def __getitem__(self, key):
    """Overrides default __getitem__ to return values from the original dict."""
    return self._def_dict[key]

  def GetTemplateValue(self, name):
    """Get the value for a name which might appear in a template.

    Args:
      name: (str) name of the value.
    Returns:
      object or None if not found.
    """
    return self._def_dict.get(name, None)

  # pylint: disable=unused-argument
  def SetTemplateValue(self, name, value, meaning=None):
    """Adds a name/value pair to the template."""
    self._def_dict[name] = value
    # TODO(user): call something like docmaker.add(
    #    self.__class__.__name__, name, meaning)

  def DeleteTemplateValue(self, name):
    """Delete a value from the object."""
    if name in self._def_dict:
      del self._def_dict[name]

  @property
  def values(self):
    """Return the underlying name/value pair dictionary."""
    return self._def_dict

  @property
  def raw(self):
    return self._raw_def_dict

  def get(self, key, default=None):  # pylint:disable=g-bad-name
    return self._def_dict.get(key, default)


class CodeObject(UseableInTemplates):
  """Template objects which represents an element that might be in code.

  This is the base class for things which might be code elements, such as
  classes, variables and methods.
  """

  _validator = name_validator

  def __init__(self, def_dict, api, parent=None, wire_name=None,
               language_model=None):
    """Construct a CodeObject.

    Args:
      def_dict: (dict) The discovery dictionary for this element.
      api: (Api) The Api instance which owns this element.
      parent: (CodeObject) The parent of this element.
      wire_name: (str) The name of this object as it appears in the protocol.
      language_model: (LanguageModel) The language we are targetting.
        Dynamically defaults to the parent's language model.
    """
    super(CodeObject, self).__init__(def_dict)
    self._api = api
    self._children = []
    self._parent = None
    self._language_model = language_model
    self._module = None
    if wire_name:
      self.SetTemplateValue(
          'wireName',
          wire_name,
          meaning='The name of this object as it appears in the data stream.')
    self.SetParent(parent)
    # Sanitize the 'description'. It is a block of user written text we want to
    # emit whenever possible.
    d = def_dict.get('description')
    if d:
      self.SetTemplateValue('description',
                            self.ValidateAndSanitizeComment(self.StripHTML(d)))

  @classmethod
  def ValidateName(cls, name):
    """Validate that the name is safe to use in generated code."""
    cls._validator.Validate(name)

  @classmethod
  def ValidateAndSanitizeComment(cls, comment):
    """Remove unsafe constructions from a string and make it safe in templates.

    Make sure a string intended as a comment has only safe constructs in it and
    then make it as safe to expand directly in a template. If it fails the test,
    return an empty string.

    Args:
      comment: (str) A string which is expected to be a documentation comment.

    Returns:
      (str) The comment with HTML-unsafe constructions removed.
    """
    return MarkSafe(cls._validator.ValidateAndSanitizeComment(comment))

  @staticmethod
  def StripHTML(input_string):
    """Strip HTML from a string."""
    stripper = html_stripper.HTMLStripper()
    stripper.feed(input_string)
    return stripper.GetFedData()

  @property
  def api(self):
    return self._api

  @property
  def children(self):
    return self._children

  @property
  def parent(self):
    return self._parent

  @property
  def module(self):
    """Returns the module this object would belong in.

    Walks up the ancesters _module is undefined for self. If a module can not
    be found, raises an error, since this indicates either a problem building
    the API model or in writing a template. Failing silently does not help
    debug that.

    Returns:
      Module

    Raises:
      ValueError: If the module can not be determined.
    """
    if self._module:
      return self._module
    if self.parent:
      return self.parent.module
    if self.api:
      return self.api.module
    raise ValueError('Asked for module of CodeObject without any: %s, %s' %
                     (self.values.get('wireName', '<unnamed>'), self))

  @property
  def codeName(self):  # pylint: disable=g-bad-name
    """Returns a language appropriate name for this object.

    This property should only be used during template expansion. It is computed
    once, using the LanguageModel in play, and then that value is cached.

    Returns:
      (str) a name for an instance of this object.
    """
    # Note that if code name is in self._def_dict (and hence GetTemplateValue
    # returns something) then this property won't be called during template
    # expansion at all.  Therefore, the change this makes -- marking codeName
    # safe -- may never take place at all.
    code_name = self.GetTemplateValue('codeName')
    if not code_name:
      code_name = self.values['wireName']
      if self.language_model:
        code_name = self.language_model.ToMemberName(code_name, self._api)
    code_name = MarkSafe(code_name)
    self.SetTemplateValue('codeName', code_name)
    return code_name

  @property
  def fullClassName(self):  # pylint: disable=g-bad-name
    """Returns the fully qualified class name for this object.

    This property can only be used during template expansion.  Walks up the
    parent chain building a fully qualified class name. If the object is in a
    module, include the module name.

    Returns:
      (str) The class name of this object.
    """
    module = self.module
    if module:
      class_name_delimiter = self.language_model.class_name_delimiter
      return module.name + class_name_delimiter + self.RelativeClassName(None)
    return MarkSafe(self.RelativeClassName(None))

  @property
  def packageRelativeClassName(self):  # pylint: disable=g-bad-name
    """Returns the class name for this object relative to its package.

    Walks up the parent chain building a fully qualified class name.

    Returns:
      (str) The class name of this object.
    """
    return MarkSafe(self.RelativeClassName(None))

  def RelativeClassName(self, other):
    """Returns the class name for this object relative to another.

    This property can only be used during template expansion.

    Args:
      other: (CodeObject) Another code object which might be a parent.
    Returns:
      (str) The class name of this object relative to another.
    """
    if self == other:
      return ''
    full_name = ''
    if self.parent:
      full_name = self.parent.RelativeClassName(other)
    if full_name:
      language_model = self.language_model
      if language_model:
        class_name_delimiter = language_model.class_name_delimiter
      full_name += class_name_delimiter
    full_name += (self.values.get('className')
                  or self.values.get('codeName')
                  or self.values.get('name', ''))
    return full_name

  @property
  def parentPath(self):  # pylint: disable=g-bad-name
    """Returns the classNames from my ultimate parent to my immediate parent.

    Walks up the parent chain building a list of ancestors.

    TODO(user): Eliminate this routine by adding template tags which
    let me process the ancestor list directly.

    Returns:
      (list) The class name of this object.
    """
    parent_list = self.ancestors
    return [p.values.get('className') for p in parent_list]

  @property
  def ancestors(self):
    """Return the objects from my ultimate parent down to my immediate parent.

    Returns:
      (list) list of CodeObjects.
    """
    if self.parent:
      return self.parent.ancestors + [self.parent]
    return []

  @property
  def full_path(self):
    """Return the objects from my ultimate parent down to me.

    Returns:
      (list) list of CodeObjects.
    """
    return (self.ancestors or []) + [self]

  def FindTopParent(self):
    if self.parent:
      return self.parent.FindTopParent()
    return self

  def SetLanguageModel(self, language_model):
    """Changes the language model of this code object."""
    self._language_model = language_model

  def SetParent(self, parent):
    """Changes the parent of this code object.

    Args:
      parent: (CodeObject) the new parent.
    """
    if self._parent:
      self._parent.children.remove(self)
    self._parent = parent
    if self._parent:
      self._parent.children.append(self)

  @property
  def language_model(self):
    """Returns the nearest LanguageModel by walking my parents.

    Memoizes the computation, by setting self._language_model after the first
    parent lookup.

    Returns:
      (LanguageModel) A LanguageModel
    """
    if self._language_model:
      return self._language_model
    if self._parent:
      self._language_model = self._parent.language_model
    return self._language_model

  @property
  def codeType(self):  # pylint: disable=g-bad-name
    """Accessor for codeType for use in templates.

    If the template value for codeType was explicitly set, return that,
    otherwise use the code_type member. This is only safe to call for code
    objects which implement code_type.

    Returns:
      (str) the value for codeType
    """
    return MarkSafe(self.GetTemplateValue('codeType') or self.code_type)

  @property
  def safeCodeType(self):  # pylint: disable=g-bad-name
    """Expose this in template using the template naming convention.

    Just redirect to safe_code_type.

    Returns:
      (str) The evaluated code type.
    """
    return MarkSafe(self.safe_code_type)

  @property
  def constantName(self):  # pylint: disable=g-bad-name
    """Returns a name for this object when used as an constant."""
    return self.language_model.ApplyPolicy('constant', self,
                                           self.values['wireName'])

  @property
  def memberName(self):  # pylint: disable=g-bad-name
    """Returns a name for this object when used as an class member."""
    return self.language_model.ApplyPolicy('member', self,
                                           self.values['wireName'])

  @property
  def getterName(self):  # pylint: disable=g-bad-name
    """Returns a name for the getter of memberName."""
    return self.language_model.ApplyPolicy('getter', self,
                                           self.values['wireName'])

  @property
  def setterName(self):  # pylint: disable=g-bad-name
    """Returns a name for the setter of memberName."""
    return self.language_model.ApplyPolicy('setter', self,
                                           self.values['wireName'])

  @property
  def hasName(self):  # pylint: disable=g-bad-name
    """Returns a name for the has check of memberName."""
    return self.language_model.ApplyPolicy('has', self, self.values['wireName'])

  @property
  def unsetName(self):  # pylint: disable=g-bad-name
    """Returns a name for the unset method of memberName."""
    return self.language_model.ApplyPolicy('unset', self,
                                           self.values['wireName'])

  @property
  def parameterName(self):  # pylint: disable=g-bad-name
    """Returns a name for this object when used as the parameter to a method."""
    return self.language_model.ApplyPolicy('parameter_name', self,
                                           self.values['wireName'])


class Module(CodeObject):
  """A code object which represents the concept of a module.

  A Module has two properties available for use in templates:
    name: The full name of this module, including the parent of this Module.
    path: The file path where this module would be stored in a full generated
          code layout. Since the templates can not open files for writing, this
          is intended for use inside documentation.

  These values are derived from elements defining the owner of the API or
  shared data type.

  Typically, a code generator will create a model (e.g. an Api) and assign a
  a Module to the top node. Other nodes in the model might be in different
  modules, which can be created as children of the top Module. E.g.
    api = LoadApi(....)
    top_module = Module(... api owner information ...)
    api_module = Module(api.name, parent=top_module)
    api._module = api_module
    model_module = Module('model', parent=api_module)
    for s in api.schemas:
      s._module = model_module

  Shared data types contain information that specify the module they belong to,
  which may be different from the module for the API itself.

  Modules are mutable up until the first time the name or path properties
  are evaluated.
  """

  def __init__(self, package_path=None, owner_name=None, owner_domain=None,
               parent=None, language_model=None):
    """Construct a Module.

    Args:
      package_path: (str) A '/' delimited path to this module.
      owner_name: (str) The name of the owner of the API, as they would like it
        to appear in library code. E.g "Best Buy"
      owner_domain: (str) The domain of the owner of the API, as they would like
        it to appear in library code.
      parent: (CodeObject) The parent of this element.
      language_model: (LanguageModel) The language we are targetting.
        Dynamically defaults to the parent's language model.
    """
    super(Module, self).__init__({}, None,
                                 parent=parent,
                                 language_model=language_model)
    self._package_path = utilities.NoSpaces(package_path)
    self._owner_name = utilities.NoSpaces(owner_name)
    self._owner_domain = utilities.SanitizeDomain(owner_domain)
    self._name = None  # will be memoized on first call to name property

  @classmethod
  def ModuleFromDictionary(cls, def_dict):
    """Returns a Module corresponding the library_definition of an object.

    If there is a 'library_definition' section in the given dictionary, use it
    to construct a Module from that information. Return None if there is no
    definition.

    Args:
      def_dict: (dict) Discovery style object definition.
    Returns:
      Module or None.
    """
    lib_def = def_dict.get('library_definition')
    if not lib_def:
      return None
    # Newer style uses modulePath, but some paths through Discovery may use
    # packagePath instead.
    return Module(package_path=(lib_def.get('modulePath')
                                or lib_def.get('packagePath')
                                or ''),
                  owner_name=lib_def.get('owner'),
                  owner_domain=lib_def.get('domain'))

  def SetPath(self, path):
    """Changes the path for this module.

    May be called up until the first time we ask for the module name. This
    restriction is to detect a class of coding errors which could occur if we
    incorrectly share the Module across types of different parentage.

    Args:
      path: (str) Path for this module ('/' delimited).

    Raises:
      ValueError: if called after the name or path properties have been evaled.
    """
    if self._name:
      raise ValueError('SetPath called after first use of name property')
    self._package_path = path

  @property
  def owner_domain(self):
    return self._owner_domain

  @property
  def owner_name(self):
    return self._owner_name

  @property
  def package_path(self):
    return self._package_path

  @property
  def name(self):
    """Returns the language appropriate name for a module."""
    if not self._name:
      self._name = self.path.replace(
          '/', self.language_model.module_name_delimiter)
    return self._name

  @property
  def path(self):
    """Returns the full / delimited file path for this package."""
    if self.parent:
      base_path = self.parent.path
    else:
      base_path = self.language_model.DefaultContainerPathForOwner(self)
    return '/'.join(x for x in (base_path, self._package_path) if x)


class Constant(CodeObject):
  """A code object which represents a constant value.

  Constants have a value and, optionally, a name and description.  The name of
  a constant is the identifier we would use in a program..  We typically use
  constants to represent the possible values of an Enum data type.
  """

  def __init__(self, value, name=None, description=None,
               parent=None, language_model=None):
    """Construct a Module.

    Args:
      value: (str|int) The string value of the constant.
      name: (str) The name for this value. If not specified, the value will be
        used as the base for the name, but numbers will be prefixed with the
        string "value_" to turn them into a valid identifier.
      description: (str) A description of the meaning of this constant.
      parent: (CodeObject) The parent of this element.
      language_model: (LanguageModel) The language we are targetting.
        Dynamically defaults to the parent's language model.
    """
    super(Constant, self).__init__({}, None, parent=parent,
                                   language_model=language_model)
    self._value = str(value)
    self.SetTemplateValue('wireName', self._value)
    if description:
      self._description = self.ValidateAndSanitizeComment(
          self.StripHTML(description))
    else:
      self._description = None
    self._name = name

  @property
  def description(self):
    return self._description

  @property
  def name(self):
    if not self._name:
      # No name, we have to make one.
      self._name = self._NameFromValue(self.value)
    return self._name

  @property
  def value(self):
    return self._value

  @property
  def constantName(self):  # pylint: disable=g-bad-name
    """Override."""
    return self.language_model.ApplyPolicy('constant', self, self.name)

  @classmethod
  def _NameFromValue(cls, value):
    """Construct a safe name for a constant from a value.

    Constants might be numbers or strings with symbols that cannot be used in
    identifiers. We want to do the minimal transform we can to make a name
    that could be turned into an identifier.

    Args:
      value: (str) The value to derive a name from.

    Returns:
      (str): A name which could
    """
    # Many string constants begin with punctuation or symbols like '@'. Remove
    # those.
    name = value
    while not name[0].isalnum():
      name = name[1:]
      if not name:
        # we ran out? Just revert
        name = value
        break
    # If we are left with a number, we have to turn it into an alphanumeric.
    if not name[0].isalpha():
      name = 'value_' + name
    return name



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


__author__ = 'aiuto@google.com (Tony Aiuto)'



from google.apputils import basetest
from googleapis.codegen import data_types
from googleapis.codegen import language_model
from googleapis.codegen import template_objects


class DataTypesTest(basetest.TestCase):

  def testVoidDataTypeDefault(self):
    api = template_objects.CodeObject({}, None)
    void = data_types.Void(api)
    api.SetLanguageModel(language_model.LanguageModel())
    self.assertEquals('void', void.code_type)

  def testVoidDataTypeOverride(self):
    class FakeLM(language_model.LanguageModel):
      def CodeTypeForVoid(self):
        return 'the absence of all'

    api = template_objects.CodeObject({}, None)
    void = data_types.Void(api)
    api.SetLanguageModel(FakeLM())
    self.assertEquals('the absence of all', void.code_type)


if __name__ == '__main__':
  basetest.main()
